@@ -1,17 +1,20 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package groovy.util;
 
@@ -21,21 +24,19 @@ import groovy.lang.GroovyRuntimeException;
 import groovy.lang.MetaExpandoProperty;
 import groovy.lang.MissingPropertyException;
 
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
-import java.util.List;
-import java.util.ArrayList;
-import java.util.Iterator;
 
 
 /**
  * Represents a dynamically expandable bean.
- * 
+ *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  * @author Hein Meling
  * @author Pilho Kim
- * @version $Revision: 17834 $
  */
 public class Expando extends GroovyObjectSupport {
 
@@ -61,19 +62,18 @@ public class Expando extends GroovyObjectSupport {
     public List getMetaPropertyValues() {
         // run through all our current properties and create MetaProperty objects
         List ret = new ArrayList();
-        Iterator itr = getProperties().entrySet().iterator();
-        while(itr.hasNext()) {
-            Entry entry = (Entry) itr.next();
+        for (Object o : getProperties().entrySet()) {
+            Entry entry = (Entry) o;
             ret.add(new MetaExpandoProperty(entry));
         }
-		
+
         return ret;
     }
 
     public Object getProperty(String property) {
         // always use the expando properties first
         Object result = getProperties().get(property);
-        if (result!=null) return result;
+        if (result != null) return result;
         try {
             return super.getProperty(property);
         }
@@ -100,21 +100,20 @@ public class Expando extends GroovyObjectSupport {
                 closure = (Closure) closure.clone();
                 closure.setDelegate(this);
                 return closure.call((Object[]) args);
-            }
-            else {
+            } else {
                 throw e;
             }
         }
-        
+
     }
-    
+
     /**
      * This allows toString to be overridden by a closure <i>field</i> method attached
      * to the expando object.
-     * 
+     *
      * @see java.lang.Object#toString()
      */
-     public String toString() {
+    public String toString() {
         Object method = getProperties().get("toString");
         if (method != null && method instanceof Closure) {
             // invoke overridden toString closure method
@@ -124,7 +123,7 @@ public class Expando extends GroovyObjectSupport {
         } else {
             return expandoProperties.toString();
         }
-     }
+    }
 
     /**
      * This allows equals to be overridden by a closure <i>field</i> method attached
@@ -166,6 +165,7 @@ public class Expando extends GroovyObjectSupport {
 
     /**
      * Factory method to create a new Map used to store the expando properties map
+     *
      * @return a newly created Map implementation
      */
     protected Map createMap() {
