@@ -1,32 +1,35 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.codehaus.groovy.antlr.treewalker;
 
-import java.io.PrintStream;
-import java.util.Stack;
-
+import antlr.collections.AST;
 import org.codehaus.groovy.antlr.GroovySourceAST;
 import org.codehaus.groovy.antlr.parser.GroovyTokenTypes;
 
+import java.io.PrintStream;
+import java.util.Stack;
+
 /**
  * An antlr AST visitor that prints groovy source code for each visited node
  * to the supplied PrintStream.
  *
  * @author <a href="mailto:groovy@ross-rayner.com">Jeremy Rayner</a>
- * @version $Revision: 16708 $
  */
 
 public class SourcePrinter extends VisitorAdapter {
@@ -64,110 +67,110 @@ public class SourcePrinter extends VisitorAdapter {
     }
     
 
-	public void visitAbstract(GroovySourceAST t, int visit) {
-		print(t,visit,"abstract ",null,null);
-	}
+    public void visitAbstract(GroovySourceAST t, int visit) {
+        print(t,visit,"abstract ",null,null);
+    }
 
-	public void visitAnnotation(GroovySourceAST t, int visit) {
-		if (visit == OPENING_VISIT) {
-			print(t,visit,"@");
-		}
-		if (visit == SECOND_VISIT) {
-			print(t,visit,"(");
-		}
-		if (visit == SUBSEQUENT_VISIT) {
-			print(t,visit,", ");
-		}
-		if (visit == CLOSING_VISIT) {
-			if (t.getNumberOfChildren() > 1) {
-				print(t,visit,") ");
-			} else {
-				print(t,visit," ");
-			}
-		}
+    public void visitAnnotation(GroovySourceAST t, int visit) {
+        if (visit == OPENING_VISIT) {
+            print(t,visit,"@");
+        }
+        if (visit == SECOND_VISIT) {
+            print(t,visit,"(");
+        }
+        if (visit == SUBSEQUENT_VISIT) {
+            print(t,visit,", ");
+        }
+        if (visit == CLOSING_VISIT) {
+            if (t.getNumberOfChildren() > 1) {
+                print(t,visit,") ");
+            } else {
+                print(t,visit," ");
+            }
+        }
 
     }
 
     public void visitAnnotations(GroovySourceAST t, int visit) {
-    	// do nothing
+        // do nothing
     }
 
     public void visitAnnotationDef(GroovySourceAST t,int visit) {
         print(t,visit,"@interface ",null,null);
     }
 
-	public void visitAnnotationFieldDef(GroovySourceAST t, int visit) {
-    	print(t,visit,"() ","default ",null);
-	}
+    public void visitAnnotationFieldDef(GroovySourceAST t, int visit) {
+        print(t,visit,"() ","default ",null);
+    }
 
-	public void visitAnnotationMemberValuePair(GroovySourceAST t, int visit) {
-		print(t,visit," = ",null,null);
-	}
-
-	public void visitArrayDeclarator(GroovySourceAST t, int visit) {
-		//<ARRAY_DECLARATOR>int</ARRAY_DECLARATOR> primes = new int(<ARRAY_DECLARATOR>5</ARRAY_DECLARATOR>)
-		if (getParentNode().getType() == GroovyTokenTypes.TYPE ||
-				getParentNode().getType() == GroovyTokenTypes.TYPECAST) { // ugly hack
-			// type definition, i.e.   int[] x;
-			print(t,visit,null,null,"[]");
-		} else {
-			// usually in new, i.e.   def y = new int[5];
-			print(t,visit,"[",null,"]");
-		}
-	}
-
-	public void visitAssign(GroovySourceAST t,int visit) {
+    public void visitAnnotationMemberValuePair(GroovySourceAST t, int visit) {
         print(t,visit," = ",null,null);
     }
-	
+
+    public void visitArrayDeclarator(GroovySourceAST t, int visit) {
+        //<ARRAY_DECLARATOR>int</ARRAY_DECLARATOR> primes = new int(<ARRAY_DECLARATOR>5</ARRAY_DECLARATOR>)
+        if (getParentNode().getType() == GroovyTokenTypes.TYPE ||
+                getParentNode().getType() == GroovyTokenTypes.TYPECAST) { // ugly hack
+            // type definition, i.e.   int[] x;
+            print(t,visit,null,null,"[]");
+        } else {
+            // usually in new, i.e.   def y = new int[5];
+            print(t,visit,"[",null,"]");
+        }
+    }
+
+    public void visitAssign(GroovySourceAST t,int visit) {
+        print(t,visit," = ",null,null);
+    }
+    
     // visitAt() ...
     //   token type 'AT' should never be visited, as annotation definitions and usage, and
     //   direct field access should have all moved this token out of the way. No test needed.
 
-	//   one of the BAND tokens is actually replaced by TYPE_UPPER_BOUNDS (e.g. class Foo<T extends C & I> {T t} )
+    //   one of the BAND tokens is actually replaced by TYPE_UPPER_BOUNDS (e.g. class Foo<T extends C & I> {T t} )
     public void visitBand(GroovySourceAST t, int visit) {
         print(t,visit," & ",null,null);
     }
 
-	public void visitBandAssign(GroovySourceAST t,int visit) {
+    public void visitBandAssign(GroovySourceAST t,int visit) {
         print(t,visit," &= ",null,null);
     }
-	
-    // visitBigSuffix() ...
-	//   token type BIG_SUFFIX never created/visited, NUM_BIG_INT, NUM_BIG_DECIMAL instead...    
     
-	// visitBlock() ...
-	//   token type BLOCK never created/visited, see CLOSABLE_BLOCK etc...
-	
-	public void visitBnot(GroovySourceAST t, int visit) {
-		print(t,visit,"~",null,null);
-	}
-	
-	// Note: old closure syntax using BOR is deprecated, and also never creates/visits a BOR node
+    // visitBigSuffix() ...
+    //   token type BIG_SUFFIX never created/visited, NUM_BIG_INT, NUM_BIG_DECIMAL instead...    
+    
+    // visitBlock() ...
+    //   token type BLOCK never created/visited, see CLOSABLE_BLOCK etc...
+    
+    public void visitBnot(GroovySourceAST t, int visit) {
+        print(t,visit,"~",null,null);
+    }
+    
+    // Note: old closure syntax using BOR is deprecated, and also never creates/visits a BOR node
     public void visitBor(GroovySourceAST t, int visit) {
         print(t,visit," | ",null,null);
     }
-	
-	public void visitBorAssign(GroovySourceAST t,int visit) {
+    
+    public void visitBorAssign(GroovySourceAST t,int visit) {
         print(t,visit," |= ",null,null);
     }
-	
+    
     public void visitBsr(GroovySourceAST t, int visit) {
         print(t,visit," >>> ",null,null);
     }
-	
-	public void visitBsrAssign(GroovySourceAST t,int visit) {
+    
+    public void visitBsrAssign(GroovySourceAST t,int visit) {
         print(t,visit," >>>= ",null,null);
     }
-	
+    
     public void visitBxor(GroovySourceAST t, int visit) {
         print(t,visit," ^ ",null,null);
     }
-	
-	public void visitBxorAssign(GroovySourceAST t,int visit) {
+    
+    public void visitBxorAssign(GroovySourceAST t,int visit) {
         print(t,visit," ^= ",null,null);
     }
-	
+    
     public void visitCaseGroup(GroovySourceAST t, int visit) {
         if (visit == OPENING_VISIT) {
             tabLevel++;
@@ -191,11 +194,11 @@ public class SourcePrinter extends VisitorAdapter {
     }
     
     public void visitClosureList(GroovySourceAST t, int visit) {
-    	print(t,visit,"(","; ",")");
+        print(t,visit,"(","; ",")");
     }
     // visitClosureOp ...
-	//   token type CLOSABLE_BLOCK_OP never created/visited, see CLOSABLE_BLOCK...
-	
+    //   token type CLOSABLE_BLOCK_OP never created/visited, see CLOSABLE_BLOCK...
+    
 
     // visitColon ...
     //   token type COLON never created/visited, see LABELED_STAT, FOR_IN_ITERABLE, 
@@ -219,7 +222,7 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitDec(GroovySourceAST t, int visit) {
-    	print(t,visit,"--",null,null);
+        print(t,visit,"--",null,null);
     }
     
     // visitDigit ...
@@ -229,10 +232,10 @@ public class SourcePrinter extends VisitorAdapter {
         print(t,visit," / ",null,null);
     }
 
-	public void visitDivAssign(GroovySourceAST t,int visit) {
+    public void visitDivAssign(GroovySourceAST t,int visit) {
         print(t,visit," /= ",null,null);
     }
-	
+    
     // visitDollar ...
     //   token type DOLLAR never created/visited, see SCOPE_ESCAPE instead
     
@@ -241,27 +244,27 @@ public class SourcePrinter extends VisitorAdapter {
     }
     
     public void visitDynamicMember(GroovySourceAST t, int visit) {
-    	if (t.childOfType(GroovyTokenTypes.STRING_CONSTRUCTOR) == null) {
-    		printUpdatingTabLevel(t,visit,"(",null,")");
-    	}
+        if (t.childOfType(GroovyTokenTypes.STRING_CONSTRUCTOR) == null) {
+            printUpdatingTabLevel(t,visit,"(",null,")");
+        }
     }
     
     public void visitElist(GroovySourceAST t,int visit) {
-    	if (getParentNode().getType() == GroovyTokenTypes.ENUM_CONSTANT_DEF) {
-    		print(t,visit,"(",", ",")");
-    	} else {
-    		print(t,visit,null,", ",null);
-    	}
+        if (getParentNode().getType() == GroovyTokenTypes.ENUM_CONSTANT_DEF) {
+            print(t,visit,"(",", ",")");
+        } else {
+            print(t,visit,null,", ",null);
+        }
     }
 
     // visitEmptyStat ...
     //   token type EMPTY_STAT obsolete and should be removed, never visited/created
     
     public void visitEnumConstantDef(GroovySourceAST t,int visit) {
-    	GroovySourceAST sibling = (GroovySourceAST)t.getNextSibling();
-    	if (sibling != null && sibling.getType() == GroovyTokenTypes.ENUM_CONSTANT_DEF) {
-    		print(t,visit,null,null,", ");
-    	}
+        GroovySourceAST sibling = (GroovySourceAST)t.getNextSibling();
+        if (sibling != null && sibling.getType() == GroovyTokenTypes.ENUM_CONSTANT_DEF) {
+            print(t,visit,null,null,", ");
+        }
     }
 
     public void visitEnumDef(GroovySourceAST t,int visit) {
@@ -279,7 +282,7 @@ public class SourcePrinter extends VisitorAdapter {
     //   token type EXPONENT only used by lexer, never visited/created
     
     public void visitExpr(GroovySourceAST t,int visit) {
-    	// do nothing
+        // do nothing
     }
 
     public void visitExtendsClause(GroovySourceAST t,int visit) {
@@ -290,21 +293,21 @@ public class SourcePrinter extends VisitorAdapter {
         }
     }
     
-	public void visitFinal(GroovySourceAST t, int visit) {
+    public void visitFinal(GroovySourceAST t, int visit) {
         print(t,visit,"final ",null,null);
-	}
-
-	// visitFloatSuffix ... never visited/created see NUM_DOUBLE or NUM_FLOAT instead
-	
-	public void visitForCondition(GroovySourceAST t, int visit) {
-    	print(t,visit," ; ",null,null);
     }
-	
-	// visitForEachClause ... 
-	//   FOR_EACH_CLAUSE obsolete and should be removed, never visited/created
+
+    // visitFloatSuffix ... never visited/created see NUM_DOUBLE or NUM_FLOAT instead
+    
+    public void visitForCondition(GroovySourceAST t, int visit) {
+        print(t,visit," ; ",null,null);
+    }
+    
+    // visitForEachClause ... 
+    //   FOR_EACH_CLAUSE obsolete and should be removed, never visited/created
 
     public void visitForInit(GroovySourceAST t, int visit) {
-    	print(t,visit,"(",null,null);
+        print(t,visit,"(",null,null);
     }
     
     public void visitForInIterable(GroovySourceAST t, int visit) {
@@ -312,11 +315,11 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitForIterator(GroovySourceAST t, int visit) {
-    	print(t,visit," ; ",null,")");
+        print(t,visit," ; ",null,")");
     }
     
     public void visitGe(GroovySourceAST t, int visit) {
-    	print(t,visit," >= ",null,null);
+        print(t,visit," >= ",null,null);
     }
     
     public void visitGt(GroovySourceAST t, int visit) {
@@ -339,7 +342,7 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitImplicitParameters(GroovySourceAST t, int visit) {
-    	// do nothing
+        // do nothing
     }
 
     public void visitImport(GroovySourceAST t,int visit) {
@@ -347,7 +350,7 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitInc(GroovySourceAST t, int visit) {
-    	print(t,visit,"++",null,null);
+        print(t,visit,"++",null,null);
     }
 
     public void visitIndexOp(GroovySourceAST t, int visit) {
@@ -359,14 +362,14 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitInstanceInit(GroovySourceAST t, int visit) {
-    	// do nothing
-	}
+        // do nothing
+    }
 
-	public void visitLabeledArg(GroovySourceAST t, int visit) {
+    public void visitLabeledArg(GroovySourceAST t, int visit) {
         print(t,visit,":",null,null);
     }
 
-	public void visitLabeledStat(GroovySourceAST t, int visit) {
+    public void visitLabeledStat(GroovySourceAST t, int visit) {
         print(t,visit,":",null,null);
     }
 
@@ -381,7 +384,7 @@ public class SourcePrinter extends VisitorAdapter {
     //   token type LCURLY only used inside parser, never visited/created
     
     public void visitLe(GroovySourceAST t, int visit) {
-    	print(t,visit," <= ",null,null);
+        print(t,visit," <= ",null,null);
     }
 
     // visitLetter ...
@@ -396,11 +399,11 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitLiteralAssert(GroovySourceAST t,int visit) {
-    	if (t.getNumberOfChildren() > 1) {
-    		print(t,visit,"assert ",null," : ");
-    	} else {
-    		print(t,visit,"assert ",null,null);
-    	}
+        if (t.getNumberOfChildren() > 1) {
+            print(t,visit,"assert ",null," : ");
+        } else {
+            print(t,visit,"assert ",null,null);
+        }
     }
 
     public void visitLiteralBoolean(GroovySourceAST t, int visit) {
@@ -420,7 +423,7 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitLiteralCatch(GroovySourceAST t,int visit) {
-        printUpdatingTabLevel(t,visit," catch (",null,") ");
+        printUpdatingTabLevel(t, visit, " catch (", null, ") ");
     }
 
     public void visitLiteralChar(GroovySourceAST t, int visit) {
@@ -503,12 +506,12 @@ public class SourcePrinter extends VisitorAdapter {
         print(t,visit,"native ",null,null);
     }
     public void visitLiteralNew(GroovySourceAST t,int visit) {
-    	if (t.childOfType(GroovyTokenTypes.ARRAY_DECLARATOR) == null) {
-    		// only print parenthesis if is not of form def x = new int[5]
-    		print(t,visit,"new ","(",")");
-    	} else {
-    		print(t,visit,"new ",null,null);
-    	}
+        if (t.childOfType(GroovyTokenTypes.ARRAY_DECLARATOR) == null) {
+            // only print parenthesis if is not of form def x = new int[5]
+            print(t,visit,"new ","(",")");
+        } else {
+            print(t,visit,"new ",null,null);
+        }
     }
 
     public void visitLiteralNull(GroovySourceAST t, int visit) {
@@ -543,8 +546,8 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitLiteralSuper(GroovySourceAST t, int visit) {
-    	// only visited when calling super() without parentheses, i.e. "super 99" is equivalent to "super(99)"
-    	print(t,visit,"super",null,null);
+        // only visited when calling super() without parentheses, i.e. "super 99" is equivalent to "super(99)"
+        print(t,visit,"super",null,null);
     }
 
     public void visitLiteralSwitch(GroovySourceAST t, int visit) {
@@ -562,12 +565,12 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitLiteralSynchronized(GroovySourceAST t,int visit) {
-    	if (t.getNumberOfChildren() > 0) {
-    		print(t,visit,"synchronized (",null,") ");
-    	} else {
-    		print(t,visit,"synchronized ",null,null);    		
-    	}
-	}
+        if (t.getNumberOfChildren() > 0) {
+            print(t,visit,"synchronized (",null,") ");
+        } else {
+            print(t,visit,"synchronized ",null,null);            
+        }
+    }
 
     public void visitLiteralThis(GroovySourceAST t, int visit) {
         print(t,visit,"this",null,null);
@@ -614,7 +617,7 @@ public class SourcePrinter extends VisitorAdapter {
         print(t,visit,"!",null,null);
     }
 
-	// Note: old closure syntax using LOR is deprecated, and also never creates/visits a LOR node
+    // Note: old closure syntax using LOR is deprecated, and also never creates/visits a LOR node
     public void visitLor(GroovySourceAST t, int visit) {
         print(t,visit," || ",null,null);
     }
@@ -636,11 +639,11 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitMethodCall(GroovySourceAST t,int visit) {
-    	if ("<command>".equals(t.getText())) {
-    		printUpdatingTabLevel(t,visit," "," ",null);
-    	} else {
-    		printUpdatingTabLevel(t,visit,"("," ",")");
-    	}
+        if ("<command>".equals(t.getText())) {
+            printUpdatingTabLevel(t,visit," "," ",null);
+        } else {
+            printUpdatingTabLevel(t,visit,"("," ",")");
+        }
     }
     public void visitMethodDef(GroovySourceAST t,int visit) {
         //do nothing
@@ -666,6 +669,22 @@ public class SourcePrinter extends VisitorAdapter {
         print(t,visit," %= ",null,null);
     }
 
+    @Override
+    public void visitMultiCatch(final GroovySourceAST t, final int visit) {
+        if (visit == CLOSING_VISIT) {
+            final AST child = t.getFirstChild();
+            if ("MULTICATCH_TYPES".equals(child.getText())) {
+                print(t, visit, null, null, " "+child.getNextSibling().getText());
+            } else {
+                print(t, visit, null, null, " "+child.getFirstChild().getText());
+            }
+        }
+    }
+
+    @Override
+    public void visitMultiCatchTypes(final GroovySourceAST t, final int visit) {
+    }
+
     // visitNls
     //   new lines are used by parser, but are not created on the AST,
     //   they can be implied by the source code line/column information
@@ -723,11 +742,11 @@ public class SourcePrinter extends VisitorAdapter {
     }
 
     public void visitParameters(GroovySourceAST t,int visit) {
-    	if (getParentNode().getType() == GroovyTokenTypes.CLOSABLE_BLOCK) {
-    		printUpdatingTabLevel(t,visit,null,","," ");
-    	} else {
-    		printUpdatingTabLevel(t,visit,"(",", ",") ");
-    	}
+        if (getParentNode().getType() == GroovyTokenTypes.CLOSABLE_BLOCK) {
+            printUpdatingTabLevel(t,visit,null,","," ");
+        } else {
+            printUpdatingTabLevel(t,visit,"(",", ",") ");
+        }
     }
 
     public void visitPlus(GroovySourceAST t, int visit) {
@@ -738,11 +757,11 @@ public class SourcePrinter extends VisitorAdapter {
         print(t,visit," += ",null,null);
     }
     public void visitPostDec(GroovySourceAST t, int visit) {
-    	print(t,visit,null,null,"--");
+        print(t,visit,null,null,"--");
     }
 
     public void visitPostInc(GroovySourceAST t, int visit) {
-    	print(t,visit,null,null,"++");
+        print(t,visit,null,null,"++");
     }
 
     public void visitQuestion(GroovySourceAST t, int visit) {
@@ -770,16 +789,16 @@ public class SourcePrinter extends VisitorAdapter {
     //    token types REGEXP_CTOR_END, REGEXP_LITERAL, REGEXP_SYMBOL only used inside lexer
     
     public void visitRegexFind(GroovySourceAST t, int visit) {
-    	print(t,visit," =~ ",null,null);
+        print(t,visit," =~ ",null,null);
     }
     public void visitRegexMatch(GroovySourceAST t, int visit) {
-    	print(t,visit," ==~ ",null,null);
+        print(t,visit," ==~ ",null,null);
     }
     // visit rparen()
     //   token type RPAREN only used inside parser, never visited/created
 
     public void visitSelectSlot(GroovySourceAST t, int visit) {
-    	print(t,visit,"@",null,null);
+        print(t,visit,"@",null,null);
     }
     
     // visit semi()
@@ -789,10 +808,10 @@ public class SourcePrinter extends VisitorAdapter {
     //  never visited/created by parser
     
     public void visitSl(GroovySourceAST t, int visit) {
-    	print(t,visit," << ",null,null);
+        print(t,visit," << ",null,null);
     }
     public void visitSlAssign(GroovySourceAST t, int visit) {
-    	print(t,visit," <<= ",null,null);
+        print(t,visit," <<= ",null,null);
     }
     public void visitSlist(GroovySourceAST t,int visit) {
         if (visit == OPENING_VISIT) {
@@ -808,45 +827,45 @@ public class SourcePrinter extends VisitorAdapter {
     //   never visited/created by parser
     
     public void visitSpreadArg(GroovySourceAST t,int visit) {
-    	print(t,visit,"*",null,null);
+        print(t,visit,"*",null,null);
     }
 
     public void visitSpreadDot(GroovySourceAST t,int visit) {
-	print(t,visit,"*.",null,null);
+    print(t,visit,"*.",null,null);
     }
 
     public void visitSpreadMapArg(GroovySourceAST t,int visit) {
-    	print(t,visit,"*:",null,null);
+        print(t,visit,"*:",null,null);
     }
     
     public void visitSr(GroovySourceAST t, int visit) {
-    	print(t,visit," >> ",null,null);
+        print(t,visit," >> ",null,null);
     }
     public void visitSrAssign(GroovySourceAST t, int visit) {
-    	print(t,visit," >>= ",null,null);
+        print(t,visit," >>= ",null,null);
     }
 
     public void visitStar(GroovySourceAST t,int visit) {
         print(t,visit,"*",null,null);
     }
     public void visitStarAssign(GroovySourceAST t, int visit) {
-    	print(t,visit," *= ",null,null);
+        print(t,visit," *= ",null,null);
     }
     public void visitStarStar(GroovySourceAST t,int visit) {
         print(t,visit,"**",null,null);
     }
     public void visitStarStarAssign(GroovySourceAST t, int visit) {
-    	print(t,visit," **= ",null,null);
+        print(t,visit," **= ",null,null);
     }
     
     public void visitStaticInit(GroovySourceAST t, int visit) {
-    	print(t,visit,"static ",null,null);
+        print(t,visit,"static ",null,null);
     }
     public void visitStaticImport(GroovySourceAST t,int visit) {
         print(t,visit,"import static ",null,null);
     }
     public void visitStrictfp(GroovySourceAST t,int visit) {
-    	print(t,visit,"strictfp ",null,null);
+        print(t,visit,"strictfp ",null,null);
     }
 
     // visitStringch
@@ -881,16 +900,25 @@ public class SourcePrinter extends VisitorAdapter {
         }
     }
 
-    private String escape(String literal) {
+    private static String escape(String literal) {
         literal = literal.replaceAll("\n","\\\\<<REMOVE>>n"); // can't seem to do \n in one go with Java regex
         literal = literal.replaceAll("<<REMOVE>>","");
         return literal;
     }
 
     public void visitSuperCtorCall(GroovySourceAST t,int visit) {
-		printUpdatingTabLevel(t,visit,"super("," ",")");
+        printUpdatingTabLevel(t,visit,"super("," ",")");
     }
-    
+
+    public void visitTraitDef(GroovySourceAST t,int visit) {
+        print(t,visit,"trait ",null,null);
+
+        if (visit == OPENING_VISIT) {
+            // store name of class away for use in constructor ident
+            className = t.childOfType(GroovyTokenTypes.IDENT).getText();
+        }
+    }
+
     // visit TripleDot, not used in the AST
     
     public void visitType(GroovySourceAST t,int visit) {
@@ -902,7 +930,7 @@ public class SourcePrinter extends VisitorAdapter {
 
             if (visit == OPENING_VISIT) {
                 if (t.getNumberOfChildren() == 0 && 
-                		parent.getType() != GroovyTokenTypes.PARAMETER_DEF) { // no need for 'def' if in a parameter list
+                        parent.getType() != GroovyTokenTypes.PARAMETER_DEF) { // no need for 'def' if in a parameter list
                     print(t,visit,"def");
                 }
             } 
@@ -916,23 +944,23 @@ public class SourcePrinter extends VisitorAdapter {
                 }
             }
             
-        	/*if (visit == CLOSING_VISIT) {
-        		print(t,visit," ");
+            /*if (visit == CLOSING_VISIT) {
+                print(t,visit," ");
             }*/
         } else {
-        	if (visit == CLOSING_VISIT) {
-        		if (t.getNumberOfChildren() != 0) {
-        			print(t,visit," ");
-        		}
-        	}
+            if (visit == CLOSING_VISIT) {
+                if (t.getNumberOfChildren() != 0) {
+                    print(t,visit," ");
+                }
+            }
         }
     }
     public void visitTypeArgument(GroovySourceAST t, int visit) {
-    	// print nothing
+        // print nothing
     }
 
     public void visitTypeArguments(GroovySourceAST t, int visit) {
-    	print(t,visit,"<",", ",">");
+        print(t,visit,"<",", ",">");
     }
 
     public void visitTypecast(GroovySourceAST t,int visit) {
@@ -942,21 +970,21 @@ public class SourcePrinter extends VisitorAdapter {
         print(t,visit," super "," & ",null);
     }
     public void visitTypeParameter(GroovySourceAST t, int visit) {
-    	// print nothing
+        // print nothing
     }
 
     public void visitTypeParameters(GroovySourceAST t, int visit) {
-    	print(t,visit,"<",", ",">");
+        print(t,visit,"<",", ",">");
     }
 
     public void visitTypeUpperBounds(GroovySourceAST t,int visit) {
         print(t,visit," extends "," & ",null);
     }
     public void visitUnaryMinus(GroovySourceAST t, int visit) {
-    	print(t,visit,"-",null,null);
+        print(t,visit,"-",null,null);
     }
     public void visitUnaryPlus(GroovySourceAST t, int visit) {
-    	print(t,visit,"+",null,null);
+        print(t,visit,"+",null,null);
     }
 
     // visit Unused "const", "do", "goto" - unsurprisingly these are unused by the AST.
@@ -973,7 +1001,7 @@ public class SourcePrinter extends VisitorAdapter {
     // visit Vocab - only used by Lexer
     
     public void visitWildcardType(GroovySourceAST t, int visit) {
-    	print(t,visit,"?",null,null);
+        print(t,visit,"?",null,null);
     }
 
     // visit WS - only used by lexer
