@@ -1,23 +1,23 @@
 /*
- * Copyright 2003-2011 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
-
 package org.codehaus.groovy.ast;
 
-import static org.codehaus.groovy.ast.ClassHelper.GROOVY_OBJECT_TYPE;
-
 import org.codehaus.groovy.ast.tools.GenericsUtils;
 import org.codehaus.groovy.ast.tools.WideningCategories;
 
@@ -26,6 +26,8 @@ import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
 
+import static org.codehaus.groovy.ast.ClassHelper.GROOVY_OBJECT_TYPE;
+
 /**
  * This class is used to describe generic type signatures for ClassNodes.
  *
@@ -71,21 +73,21 @@ public class GenericsType extends ASTNode {
 
     private String toString(Set<String> visited) {
         if (placeholder) visited.add(name);
-        String ret = wildcard?"?":((type == null || placeholder) ? name : genericsBounds(type, visited));
+        StringBuilder ret = new StringBuilder(wildcard ? "?" : ((type == null || placeholder) ? name : genericsBounds(type, visited)));
         if (upperBounds != null) {
             if (placeholder && upperBounds.length==1 && !upperBounds[0].isGenericsPlaceHolder() && upperBounds[0].getName().equals("java.lang.Object")) {
                 // T extends Object should just be printed as T
             } else {
-                ret += " extends ";
+                ret.append(" extends ");
                 for (int i = 0; i < upperBounds.length; i++) {
-                    ret += genericsBounds(upperBounds[i], visited);
-                    if (i + 1 < upperBounds.length) ret += " & ";
+                    ret.append(genericsBounds(upperBounds[i], visited));
+                    if (i + 1 < upperBounds.length) ret.append(" & ");
                 }
             }
         } else if (lowerBound != null) {
-            ret += " super " + genericsBounds(lowerBound, visited);
+            ret.append(" super ").append(genericsBounds(lowerBound, visited));
         }
-        return ret;
+        return ret.toString();
     }
 
     private String nameOf(ClassNode theType) {
@@ -279,6 +281,8 @@ public class GenericsType extends ASTNode {
                     // but with reversed arguments
                     return implementsInterfaceOrIsSubclassOf(lowerBound, classNode) && checkGenerics(classNode);
                 }
+                // If there are no bounds, the generic type is basically Object, and everything is compatible.
+                return true;
             }
             // if this is not a generics placeholder, first compare that types represent the same type
             if ((type!=null && !type.equals(classNode))) {
@@ -373,8 +377,6 @@ public class GenericsType extends ASTNode {
             for (int i = 0; redirectBoundGenericTypes!=null && i < redirectBoundGenericTypes.length && match; i++) {
                 GenericsType redirectBoundType = redirectBoundGenericTypes[i];
                 GenericsType classNodeType = cnTypes[i];
-                // The following code has been commented out because it causes GROOVY-5415
-                // However, commenting doesn't make any test fail, which is curious...
                 if (classNodeType.isPlaceholder()) {
                     String name = classNodeType.getName();
                     if (redirectBoundType.isPlaceholder()) {
@@ -437,17 +439,15 @@ public class GenericsType extends ASTNode {
                                                         gt = classNodePlaceholders.get(gt.getName());
                                                     }
                                                 }
-                                                match = match &&
-                                                        (implementsInterfaceOrIsSubclassOf(classNodeType.getType(), gt.getType())
-                                                         || classNodeType.isCompatibleWith(gt.getType())); // workaround for GROOVY-6095
+                                                match = implementsInterfaceOrIsSubclassOf(classNodeType.getType(), gt.getType())
+                                                         || classNodeType.isCompatibleWith(gt.getType()); // workaround for GROOVY-6095
                                                 if (!match) break;
                                             }
                                         }
                                         return match;
-                                    } else {
+                                    } else if (classNodePlaceholders.containsKey(name)) {
                                         redirectBoundType = classNodePlaceholders.get(name);
                                     }
-
                                 }
                             }
                             match = redirectBoundType.isCompatibleWith(classNodeType.getType());
@@ -458,8 +458,7 @@ public class GenericsType extends ASTNode {
                     }
                 }
             }
-            if (!match) return false;
-            return true;
+            return match;
         }
     }
 
