@@ -1,23 +1,23 @@
 /*
- * Copyright 2003-2010 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.codehaus.groovy.classgen.asm;
 
-import java.util.HashMap;
-import java.util.Map;
-
 import org.codehaus.groovy.ast.ClassHelper;
 import org.codehaus.groovy.ast.ClassNode;
 import org.codehaus.groovy.ast.DynamicVariable;
@@ -29,15 +29,33 @@ import org.codehaus.groovy.ast.expr.VariableExpression;
 import org.codehaus.groovy.classgen.AsmClassGenerator;
 import org.codehaus.groovy.runtime.BytecodeInterface8;
 
-import static org.codehaus.groovy.ast.ClassHelper.*;
-import static org.codehaus.groovy.syntax.Types.*;
-import static org.codehaus.groovy.ast.tools.WideningCategories.*;
+import java.util.HashMap;
+import java.util.Map;
+
+import static org.codehaus.groovy.ast.ClassHelper.boolean_TYPE;
+import static org.codehaus.groovy.ast.ClassHelper.byte_TYPE;
+import static org.codehaus.groovy.ast.ClassHelper.char_TYPE;
+import static org.codehaus.groovy.ast.ClassHelper.double_TYPE;
+import static org.codehaus.groovy.ast.ClassHelper.float_TYPE;
+import static org.codehaus.groovy.ast.ClassHelper.int_TYPE;
+import static org.codehaus.groovy.ast.ClassHelper.long_TYPE;
+import static org.codehaus.groovy.ast.ClassHelper.short_TYPE;
+import static org.codehaus.groovy.ast.tools.WideningCategories.isBigDecCategory;
+import static org.codehaus.groovy.ast.tools.WideningCategories.isDoubleCategory;
+import static org.codehaus.groovy.ast.tools.WideningCategories.isIntCategory;
+import static org.codehaus.groovy.ast.tools.WideningCategories.isLongCategory;
+import static org.codehaus.groovy.ast.tools.WideningCategories.isNumberCategory;
+import static org.codehaus.groovy.syntax.TokenUtil.removeAssignment;
+import static org.codehaus.groovy.syntax.Types.DIVIDE;
+import static org.codehaus.groovy.syntax.Types.LEFT_SHIFT;
+import static org.codehaus.groovy.syntax.Types.LEFT_SQUARE_BRACKET;
+import static org.codehaus.groovy.syntax.Types.RIGHT_SHIFT;
+import static org.codehaus.groovy.syntax.Types.RIGHT_SHIFT_UNSIGNED;
 
 /**
  * This class is for internal use only!
  * This class will dispatch to the right type adapters according to the 
  * kind of binary expression that is provided.
- * @author <a href="mailto:blackdrag@gmx.org">Jochen "blackdrag" Theodorou</a>
  */
 public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper {
     
@@ -77,18 +95,22 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
         @Override protected MethodCaller getArraySetCaller() { return shortArraySet; }    
     }
     
-    private BinaryExpressionWriter[] binExpWriter = {
-            /* 0: dummy  */ new BinaryObjectExpressionHelper(getController()),
-            /* 1: int    */ new BinaryIntExpressionHelper(getController()),
-            /* 2: long   */ new BinaryLongExpressionHelper(getController()),
-            /* 3: double */ new BinaryDoubleExpressionHelper(getController()),
-            /* 4: char   */ new BinaryCharExpressionHelper(getController()),
-            /* 5: byte   */ new BinaryByteExpressionHelper(getController()),
-            /* 6: short  */ new BinaryShortExpressionHelper(getController()),
-            /* 7: float  */ new BinaryFloatExpressionHelper(getController()),
-            /* 8: bool   */ new BinaryBooleanExpressionHelper(getController()),
-    };
-    
+    protected BinaryExpressionWriter[] binExpWriter = initializeDelegateHelpers();
+
+    protected BinaryExpressionWriter[] initializeDelegateHelpers() {
+        return new BinaryExpressionWriter[]{
+                /* 0: dummy  */ new BinaryObjectExpressionHelper(getController()),
+                /* 1: int    */ new BinaryIntExpressionHelper(getController()),
+                /* 2: long   */ new BinaryLongExpressionHelper(getController()),
+                /* 3: double */ new BinaryDoubleExpressionHelper(getController()),
+                /* 4: char   */ new BinaryCharExpressionHelper(getController()),
+                /* 5: byte   */ new BinaryByteExpressionHelper(getController()),
+                /* 6: short  */ new BinaryShortExpressionHelper(getController()),
+                /* 7: float  */ new BinaryFloatExpressionHelper(getController()),
+                /* 8: bool   */ new BinaryBooleanExpressionHelper(getController()),
+        };
+    }
+
     public static Map<ClassNode,Integer> typeMap = new HashMap<ClassNode,Integer>(14);
     static {
         typeMap.put(int_TYPE,       1); typeMap.put(long_TYPE,          2);
@@ -96,13 +118,13 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
         typeMap.put(byte_TYPE,      5); typeMap.put(short_TYPE,         6);
         typeMap.put(float_TYPE,     7); typeMap.put(boolean_TYPE,       8);
     }
-    public final static String[] typeMapKeyNames = {"dummy", "int", "long", "double", "char", "byte", "short", "float", "boolean"};
+    public static final String[] typeMapKeyNames = {"dummy", "int", "long", "double", "char", "byte", "short", "float", "boolean"};
 
     public BinaryExpressionMultiTypeDispatcher(WriterController wc) {
         super(wc);
     }
 
-    private int getOperandConversionType(ClassNode leftType, ClassNode rightType) {
+    private static int getOperandConversionType(ClassNode leftType, ClassNode rightType) {
         if (isIntCategory(leftType) && isIntCategory(rightType)) return 1;
         if (isLongCategory(leftType) && isLongCategory(rightType)) return 2;
         if (isBigDecCategory(leftType) && isBigDecCategory(rightType)) return 0;
@@ -110,34 +132,49 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
         return 0;
     }
     
-    private int getOperandType(ClassNode type) {
+    protected int getOperandType(ClassNode type) {
         Integer ret = typeMap.get(type);
         if (ret==null) return 0;
         return ret;
     }
+
+    @Deprecated
+    protected boolean doPrimtiveCompare(ClassNode leftType, ClassNode rightType, BinaryExpression binExp) {
+        return doPrimitiveCompare(leftType, rightType, binExp);
+    }
+
+    protected boolean doPrimitiveCompare(ClassNode leftType, ClassNode rightType, BinaryExpression binExp) {
+        Expression leftExp = binExp.getLeftExpression();
+        Expression rightExp = binExp.getRightExpression();
+        int operation = binExp.getOperation().getType();
+        
+        int operationType = getOperandConversionType(leftType,rightType);
+        BinaryExpressionWriter bew = binExpWriter[operationType];
+
+        if (!bew.write(operation, true)) return false;
+            
+        AsmClassGenerator acg = getController().getAcg();
+        OperandStack os = getController().getOperandStack();
+        leftExp.visit(acg);
+        os.doGroovyCast(bew.getNormalOpResultType());
+        rightExp.visit(acg);
+        os.doGroovyCast(bew.getNormalOpResultType());
+        bew.write(operation, false);
+        
+        return true;
+    }
     
     @Override
     protected void evaluateCompareExpression(final MethodCaller compareMethod, BinaryExpression binExp) {
         ClassNode current =  getController().getClassNode();
-        int operation = binExp.getOperation().getType();
+        TypeChooser typeChooser = getController().getTypeChooser();
         
         Expression leftExp = binExp.getLeftExpression();
-        ClassNode leftType = getController().getTypeChooser().resolveType(leftExp, current);
+        ClassNode leftType = typeChooser.resolveType(leftExp, current);
         Expression rightExp = binExp.getRightExpression();
-        ClassNode rightType = getController().getTypeChooser().resolveType(rightExp, current);
+        ClassNode rightType = typeChooser.resolveType(rightExp, current);
         
-        int operationType = getOperandConversionType(leftType,rightType);
-        BinaryExpressionWriter bew = binExpWriter[operationType];
-        
-        if (bew.write(operation, true)) {
-            AsmClassGenerator acg = getController().getAcg();
-            OperandStack os = getController().getOperandStack();
-            leftExp.visit(acg);
-            os.doGroovyCast(bew.getNormalOpResultType());
-            rightExp.visit(acg);
-            os.doGroovyCast(bew.getNormalOpResultType());
-            bew.write(operation, false);
-        } else {
+        if (!doPrimitiveCompare(leftType, rightType, binExp)) {
             super.evaluateCompareExpression(compareMethod, binExp);
         }
     }
@@ -168,7 +205,19 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
                 rightExp.visit(acg);
                 os.doGroovyCast(int_TYPE);
                 bew.arrayGet(operation, false);
-                os.replace(leftType,2);
+                os.replace(bew.getArrayGetResultType(),2);
+            } else {
+                super.evaluateBinaryExpression(message, binExp);
+            }
+        } else if (operation == DIVIDE) { 
+            int operationType = getOperandType(getController().getTypeChooser().resolveType(binExp, current));
+            BinaryExpressionWriter bew = binExpWriter[operationType];
+            if (bew.writeDivision(true)) {
+                leftExp.visit(acg);
+                os.doGroovyCast(bew.getDevisionOpResultType());
+                rightExp.visit(acg);
+                os.doGroovyCast(bew.getDevisionOpResultType());
+                bew.writeDivision(false);
             } else {
                 super.evaluateBinaryExpression(message, binExp);
             }
@@ -184,12 +233,6 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
                 rightExp.visit(acg);
                 os.doGroovyCast(int_TYPE);
                 bew.write(operation, false);
-            } else if (operation==DIVIDE && bew.writeDivision(true)) {
-                leftExp.visit(acg);
-                os.doGroovyCast(bew.getDevisionOpResultType());
-                rightExp.visit(acg);
-                os.doGroovyCast(bew.getDevisionOpResultType());
-                bew.writeDivision(false);
             } else if (bew.write(operation, true)) {
                 leftExp.visit(acg);
                 os.doGroovyCast(bew.getNormalOpResultType());
@@ -207,29 +250,19 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
         return isNumberCategory(type);
     }
 
-    private boolean isShiftOperation(int operation) {
+    private static boolean isShiftOperation(int operation) {
         return  operation==LEFT_SHIFT   || 
                 operation==RIGHT_SHIFT  ||
                 operation==RIGHT_SHIFT_UNSIGNED;
     }
 
-    private boolean isAssignmentToArray(BinaryExpression binExp) {
+    private static boolean isAssignmentToArray(BinaryExpression binExp) {
         Expression leftExpression = binExp.getLeftExpression();
         if (!(leftExpression instanceof BinaryExpression)) return false;
         BinaryExpression leftBinExpr = (BinaryExpression) leftExpression;
-        if (leftBinExpr.getOperation().getType() != LEFT_SQUARE_BRACKET) return false;
-        return true;
+        return leftBinExpr.getOperation().getType() == LEFT_SQUARE_BRACKET;
     }
-    
-    private int removeAssignment(int op) {
-        switch (op) {
-            case PLUS_EQUAL: return PLUS;
-            case MINUS_EQUAL: return MINUS;
-            case MULTIPLY_EQUAL: return MULTIPLY;
-            default: return op;
-        }
-    }
-    
+
     private boolean doAssignmentToArray(BinaryExpression binExp) {
         if (!isAssignmentToArray(binExp)) return false;
         // we need to handle only assignment to arrays combined with an operation
@@ -243,7 +276,7 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
         Expression rightExp = binExp.getRightExpression();
         ClassNode rightType = getController().getTypeChooser().resolveType(rightExp, current);
         
-        int operationType = getOperandConversionType(leftType,rightType);
+        int operationType = getOperandType(leftType);
         BinaryExpressionWriter bew = binExpWriter[operationType];
         
         boolean simulationSuccess = bew.arrayGet(LEFT_SQUARE_BRACKET, true);
@@ -277,6 +310,7 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
         
         // load array: load x and DUP [load sub, call arrayGet, load b, call operation, load sub, call arraySet] 
         arrayWithSubscript.getLeftExpression().visit(acg);
+        operandStack.doGroovyCast(leftType.makeArray());
         operandStack.dup();
         
         // array get: load sub, call arrayGet [load b, call operation, load sub, call arraySet]
@@ -286,17 +320,21 @@ public class BinaryExpressionMultiTypeDispatcher extends BinaryExpressionHelper
         
         // complete rhs: load b, call operation [load sub, call arraySet]
         binExp.getRightExpression().visit(acg);
+        if (! (bew instanceof BinaryObjectExpressionHelper)) {
+            // in primopts we convert to the left type for supported binary operations
+            operandStack.doGroovyCast(leftType);  
+        }
         bew.write(operation, false);
         
         // let us save that value for the return
         operandStack.dup();
-        int resultValueId = compileStack.defineTemporaryVariable("$result", rightType, true);               
+        int resultValueId = compileStack.defineTemporaryVariable("$result", rightType, true);
 
         // array set: load sub, call arraySet []
         operandStack.load(ClassHelper.int_TYPE, subscriptValueId);
         operandStack.swap();
         bew.arraySet(false);
-        operandStack.remove(2);
+        operandStack.remove(3); // 3 operands, the array, the index and the value!
 
         // load return value
         operandStack.load(rightType, resultValueId);
