@@ -1,17 +1,20 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.codehaus.groovy.reflection;
 
@@ -42,9 +45,12 @@ public class CachedConstructor extends ParameterTypes {
                     return null;
                 }
             });
-        }
-        catch (SecurityException e) {
+        } catch (SecurityException e) {
             // IGNORE
+        } catch (RuntimeException re) {
+            // test for JDK9 JIGSAW
+            if (!"java.lang.reflect.InaccessibleObjectException".equals(re.getClass().getName())) throw re;
+            // else IGNORE
         }
     }
 
@@ -76,18 +82,21 @@ public class CachedConstructor extends ParameterTypes {
         try {
             return constr.newInstance(argumentArray);
         } catch (InvocationTargetException e) {
-            throw new InvokerInvocationException(e);
+            throw e.getCause() instanceof RuntimeException ? (RuntimeException)e.getCause() : new InvokerInvocationException(e);
         } catch (IllegalArgumentException e) {
-            throw createExceptionText("failed to invoke constructor: ", constr, argumentArray, e, false);
+            throw createException("failed to invoke constructor: ", constr, argumentArray, e, false);
         } catch (IllegalAccessException e) {
-            throw createExceptionText("could not access constructor: ", constr, argumentArray, e, false);
+            throw createException("could not access constructor: ", constr, argumentArray, e, false);
         } catch (Exception e) {
-            throw createExceptionText("failed to invoke constructor: ", constr, argumentArray, e, true);
+            if (e instanceof RuntimeException)
+                throw (RuntimeException)e;
+            else
+                throw createException("failed to invoke constructor: ", constr, argumentArray, e, true);
         }
     }
 
-    private static GroovyRuntimeException createExceptionText(String init, Constructor constructor, Object[] argumentArray, Throwable e, boolean setReason) {
-        throw new GroovyRuntimeException(
+    private static GroovyRuntimeException createException(String init, Constructor constructor, Object[] argumentArray, Throwable e, boolean setReason) {
+        return new GroovyRuntimeException(
                 init
                         + constructor
                         + " with arguments: "
@@ -100,4 +109,8 @@ public class CachedConstructor extends ParameterTypes {
     public int getModifiers () {
         return cachedConstructor.getModifiers();
     }
+    
+    public CachedClass getCachedClass() {
+        return clazz;
+    }
 }
