@@ -1,25 +1,27 @@
 /*
- * Copyright 2008 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.codehaus.groovy.reflection;
 
-import java.lang.reflect.Method;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashSet;
 import java.util.Set;
-import java.util.Collections;
 
 /**
  * This class contains utility methods to determine which class called the
@@ -29,43 +31,38 @@ import java.util.Collections;
 public class ReflectionUtils {
 
     // these are packages in the call stack that are only part of the groovy MOP
-    private static final Set<String> ignoredPackages = new HashSet<String>();
+    private static final Set<String> IGNORED_PACKAGES = new HashSet<String>();
+
     static {
-        //ignoredPackages.add("java.lang.reflect");
-        ignoredPackages.add("groovy.lang");
-        ignoredPackages.add("org.codehaus.groovy.reflection");
-        ignoredPackages.add("org.codehaus.groovy.runtime.callsite");
-        ignoredPackages.add("org.codehaus.groovy.runtime.metaclass");
-        ignoredPackages.add("org.codehaus.groovy.runtime");
-        ignoredPackages.add("sun.reflect");
+        //IGNORED_PACKAGES.add("java.lang.reflect");
+        IGNORED_PACKAGES.add("groovy.lang");
+        IGNORED_PACKAGES.add("org.codehaus.groovy.reflection");
+        IGNORED_PACKAGES.add("org.codehaus.groovy.runtime.callsite");
+        IGNORED_PACKAGES.add("org.codehaus.groovy.runtime.metaclass");
+        IGNORED_PACKAGES.add("org.codehaus.groovy.runtime");
+        IGNORED_PACKAGES.add("sun.reflect");
+        IGNORED_PACKAGES.add("java.lang.invoke");
+        IGNORED_PACKAGES.add("org.codehaus.groovy.vmplugin.v7");
     }
 
-    private static final Method magicMethod;
-    static {
-        Method meth;
-        try {
-            Class srr = Class.forName("sun.reflect.Reflection");
-            meth = srr.getMethod("getCallerClass", Integer.TYPE);
-        } catch (Throwable t) {
-            meth = null;
-        }
-        magicMethod = meth;
-    }
+    private static final ClassContextHelper HELPER = new ClassContextHelper();
 
     /**
      * Determine whether or not the getCallingClass methods will return
      * any sensible results.  On JVMs that are not Sun derived i.e.
      * (gcj, Harmony) this will likely return false.  When not available
      * all getCallingClass methods will return null.
+     *
      * @return true if getCallingClass can return anything but null, false if
-     *  it will only return null.
+     *         it will only return null.
      */
     public static boolean isCallingClassReflectionAvailable() {
-        return magicMethod != null;
+        return true;
     }
 
     /**
      * Get the immediate calling class, ignoring MOP frames.
+     *
      * @return The Class of the caller
      */
     public static Class getCallingClass() {
@@ -75,10 +72,11 @@ public class ReflectionUtils {
     /**
      * Get the called that is matchLevel stack frames before the call,
      * ignoring MOP frames.
+     *
      * @param matchLevel how may call stacks down to look.
-     *      If it is less than 1 it is treated as though it was 1.
+     *                   If it is less than 1 it is treated as though it was 1.
      * @return The Class of the matched caller, or null if there aren't
-     *   enough stackframes to satisfy matchLevel
+     *         enough stackframes to satisfy matchLevel
      */
     public static Class getCallingClass(int matchLevel) {
         return getCallingClass(matchLevel, Collections.EMPTY_SET);
@@ -87,17 +85,17 @@ public class ReflectionUtils {
     /**
      * Get the called that is matchLevel stack frames before the call,
      * ignoring MOP frames and desired exclude packages.
-     * @param matchLevel how may call stacks down to look.
-     *      If it is less than 1 it is treated as though it was 1.
+     *
+     * @param matchLevel           how may call stacks down to look.
+     *                             If it is less than 1 it is treated as though it was 1.
      * @param extraIgnoredPackages A collection of string names of packages to exclude
-     *   in addition to the MOP packages when counting stack frames.
+     *                             in addition to the MOP packages when counting stack frames.
      * @return The Class of the matched caller, or null if there aren't
-     *   enough stackframes to satisfy matchLevel
+     *         enough stackframes to satisfy matchLevel
      */
     public static Class getCallingClass(int matchLevel, Collection<String> extraIgnoredPackages) {
-        if (magicMethod == null) {
-            return null;
-        }
+        Class[] classContext = HELPER.getClassContext();
+
         int depth = 0;
         try {
             Class c;
@@ -106,15 +104,15 @@ public class ReflectionUtils {
             Class sc;
             do {
                 do {
-                    c = (Class) magicMethod.invoke(null, depth++);
+                    c = classContext[depth++];
                     if (c != null) {
                         sc = c.getSuperclass();
                     } else {
                         sc = null;
                     }
                 } while (classShouldBeIgnored(c, extraIgnoredPackages)
-                    || superClassShouldBeIgnored(sc));
-            } while (c != null && matchLevel-- > 0);
+                        || superClassShouldBeIgnored(sc));
+            } while (c != null && matchLevel-- > 0 && depth<classContext.length);
             return c;
         } catch (Throwable t) {
             return null;
@@ -129,7 +127,14 @@ public class ReflectionUtils {
         return ((c != null)
                 && (c.isSynthetic()
                     || (c.getPackage() != null
-                        && (ignoredPackages.contains(c.getPackage().getName())
+                        && (IGNORED_PACKAGES.contains(c.getPackage().getName())
                           || extraIgnoredPackages.contains(c.getPackage().getName())))));
     }
+
+    private static class ClassContextHelper extends SecurityManager {
+        @Override
+        public Class[] getClassContext() {
+            return super.getClassContext();
+        }
+    }
 }
