@@ -1,25 +1,29 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.codehaus.groovy.runtime;
 
+import groovy.lang.Closure;
+import groovy.lang.GroovyObjectSupport;
+
 import java.util.Collections;
 import java.util.Iterator;
 
-import groovy.lang.GroovyObjectSupport;
-
 public class NullObject extends GroovyObjectSupport {
     private static final NullObject INSTANCE = new NullObject();
 
@@ -56,6 +60,16 @@ public class NullObject extends GroovyObjectSupport {
         throw new NullPointerException("Cannot get property '" + property + "' on null object");
     }
 
+    /**
+     * Allows the closure to be called for NullObject
+     *
+     * @param closure the closure to call on the object
+     * @return result of calling the closure
+     */
+    public <T> T with( Closure<T> closure ) {
+        return DefaultGroovyMethods.with( null, closure ) ;
+    }
+
     /**
      * Tries to set a property on null, which will always fail
      *
@@ -109,6 +123,18 @@ public class NullObject extends GroovyObjectSupport {
         return getMetaClass().invokeMethod(this, "toString", new Object[]{}) + s;
     }
 
+    /**
+     * Fallback for null+null.
+     * The result is always a NPE. The plus(String) version will catch 
+     * the case of adding a non null String to null.
+     *
+     * @param o - the Object
+     * @return nothing
+     */
+    public Object plus(Object o) {
+        throw new NullPointerException("Cannot execute null+" + String.valueOf(o));
+    }
+
     /**
      * The method "is" is used to test for equal references.
      * This method will return true only if the given parameter
@@ -131,6 +157,15 @@ public class NullObject extends GroovyObjectSupport {
         return null;
     }
 
+    /**
+     * A null object always coerces to false.
+     * 
+     * @return false
+     */
+    public boolean asBoolean() {
+        return false;
+    }
+
     public String toString() {
         return "null";
     }
