@@ -1,24 +1,27 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.codehaus.groovy.runtime.metaclass;
 
 import groovy.lang.MetaMethod;
 import org.codehaus.groovy.reflection.CachedClass;
-import org.codehaus.groovy.util.FastArray;
 import org.codehaus.groovy.reflection.GeneratedMetaMethod;
+import org.codehaus.groovy.util.FastArray;
 import org.codehaus.groovy.util.SingleKeyHashMap;
 
 import java.util.NoSuchElementException;
@@ -44,8 +47,13 @@ public class MetaMethodIndex {
     }
 
     public static class CacheEntry {
-        public Class [] params;
-        public MetaMethod method;
+        public final Class [] params;
+        public final MetaMethod method;
+
+        public CacheEntry(final Class[] params, final MetaMethod method) {
+            this.params = params;
+            this.method = method;
+        }
     }
 
     public static class Entry {
@@ -157,7 +165,6 @@ public class MetaMethodIndex {
         return new EntryIterator() {
             Entry next;    // next entry to return
             int index;        // current slot
-            Entry current;    // current entry
 
             {
                 Entry[] t = table;
@@ -191,7 +198,7 @@ public class MetaMethodIndex {
                     n = t[--i];
                 index = i;
                 next = n;
-                return current = e;
+                return e;
             }
         };
     }
@@ -308,7 +315,7 @@ public class MetaMethodIndex {
         Object oldListOrMethod = e.methodsForSuper;
         if (oldListOrMethod == null)
           return;
-        
+
         if (oldListOrMethod instanceof FastArray) {
             FastArray oldList = (FastArray) oldListOrMethod;
             int len1 = oldList.size();
@@ -339,7 +346,7 @@ public class MetaMethodIndex {
         Object oldListOrMethod = from.methods;
         if (oldListOrMethod == null)
           return;
-        
+
         if (oldListOrMethod instanceof FastArray) {
             FastArray oldList = (FastArray) oldListOrMethod;
             Entry e = null;
@@ -374,7 +381,10 @@ public class MetaMethodIndex {
                 return list;
             } else {
                 if (match.isPrivate()
-                        || (!isNonRealMethod(match) && match.getDeclaringClass().isInterface() && !method.getDeclaringClass().isInterface())) {
+                        || (!isNonRealMethod(match)
+                            && match.getDeclaringClass().isInterface()
+                            && !method.getDeclaringClass().isInterface()
+                            && !method.isStatic())) {
                     // do not overwrite interface methods with instance methods
                     // do not overwrite private methods
                     // Note: private methods from parent classes are not shown here,
@@ -406,7 +416,10 @@ public class MetaMethodIndex {
                 MetaMethod match = (MetaMethod) list.get(found);
                 if (match==method) return o;
                 if (match.isPrivate()
-                        || (!isNonRealMethod(match) && match.getDeclaringClass().isInterface() && !method.getDeclaringClass().isInterface())) {
+                        || (!isNonRealMethod(match)
+                            && match.getDeclaringClass().isInterface()
+                            && !method.getDeclaringClass().isInterface()
+                            && !method.isStatic())) {
                     // do not overwrite interface methods with instance methods
                     // do not overwrite private methods
                     // Note: private methods from parent classes are not shown here,
@@ -430,16 +443,17 @@ public class MetaMethodIndex {
         return o;
     }
 
-    private boolean isNonRealMethod(MetaMethod method) {
+    private static boolean isNonRealMethod(MetaMethod method) {
         return method instanceof NewInstanceMetaMethod ||
                 method instanceof NewStaticMetaMethod ||
                 method instanceof ClosureMetaMethod ||
                 method instanceof GeneratedMetaMethod ||
                 method instanceof ClosureStaticMetaMethod ||
-                method instanceof MixinInstanceMetaMethod;
+                method instanceof MixinInstanceMetaMethod ||
+                method instanceof ClosureMetaMethod.AnonymousMetaMethod;
     }
 
-    private boolean isMatchingMethod(MetaMethod aMethod, MetaMethod method) {
+    private static boolean isMatchingMethod(MetaMethod aMethod, MetaMethod method) {
         if (aMethod==method) return true;
         CachedClass[] params1 = aMethod.getParameterTypes();
         CachedClass[] params2 = method.getParameterTypes();
@@ -457,7 +471,7 @@ public class MetaMethodIndex {
         return matches;
     }
 
-    private int findMatchingMethod(FastArray list, MetaMethod method) {
+    private static int findMatchingMethod(FastArray list, MetaMethod method) {
         int len = list.size();
         Object data[] = list.getArray();
         for (int j = 0; j != len; ++j) {
