@@ -1,42 +1,53 @@
 /*
- * Copyright 2003-2010 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
-
 package org.codehaus.groovy.tools;
 
+import groovy.lang.Binding;
 import groovy.lang.GroovyResourceLoader;
-import org.apache.commons.cli.*;
+import groovy.lang.GroovyShell;
+import groovy.lang.GroovySystem;
+import org.apache.commons.cli.CommandLine;
+import org.apache.commons.cli.CommandLineParser;
+import org.apache.commons.cli.GroovyInternalPosixParser;
+import org.apache.commons.cli.HelpFormatter;
+import org.apache.commons.cli.OptionBuilder;
+import org.apache.commons.cli.Options;
 import org.codehaus.groovy.control.CompilationUnit;
 import org.codehaus.groovy.control.CompilerConfiguration;
 import org.codehaus.groovy.control.ConfigurationException;
+import org.codehaus.groovy.control.customizers.ImportCustomizer;
+import org.codehaus.groovy.runtime.DefaultGroovyStaticMethods;
 import org.codehaus.groovy.tools.javac.JavaAwareCompilationUnit;
 
-import groovy.lang.GroovySystem;
-
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
 import java.io.IOException;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 
 /**
  * Command-line compiler (aka. <tt>groovyc</tt>).
- *
- * @version $Id$
  */
 public class FileSystemCompiler {
     private final CompilationUnit unit;
@@ -53,13 +64,6 @@ public class FileSystemCompiler {
         } else {
             unit = new CompilationUnit(configuration);
         }
-
-        // in command line we don't need to do script lookups
-        unit.getClassLoader().setResourceLoader(new GroovyResourceLoader() {
-            public URL loadGroovySource(String filename) throws MalformedURLException {
-                return null;
-            }
-        });
     }
 
     public void compile(String[] paths) throws Exception {
@@ -80,7 +84,7 @@ public class FileSystemCompiler {
     public static void displayVersion() {
         String version = GroovySystem.getVersion();
         System.err.println("Groovy compiler version " + version);
-        System.err.println("Copyright 2003-2010 The Codehaus. http://groovy.codehaus.org/");
+        System.err.println("Copyright 2003-2019 The Apache Software Foundation. http://groovy-lang.org/");
         System.err.println("");
     }
 
@@ -112,9 +116,17 @@ public class FileSystemCompiler {
      * the VM to exit.
      */
     public static void commandLineCompile(String[] args) throws Exception {
+        commandLineCompile(args, true);
+    }
+
+    /**
+     * Same as main(args) except that exceptions are thrown out instead of causing
+     * the VM to exit and the lookup for .groovy files can be controlled
+     */
+    public static void commandLineCompile(String[] args, boolean lookupUnnamedFiles) throws Exception {
         Options options = createCompilationOptions();
 
-        PosixParser cliParser = new PosixParser();
+        CommandLineParser cliParser = new GroovyInternalPosixParser();
 
         CommandLine cli;
         cli = cliParser.parse(options, args);
@@ -145,38 +157,73 @@ public class FileSystemCompiler {
         fileNameErrors = fileNameErrors && !validateFiles(filenames);
 
         if (!fileNameErrors) {
-            doCompilation(configuration, null, filenames);
+            doCompilation(configuration, null, filenames, lookupUnnamedFiles);
         }
     }
 
     /**
      * Primary entry point for compiling from the command line
      * (using the groovyc script).
-     *
+     * <p>
      * If calling inside a process and you don't want the JVM to exit on an
-     * error call commandLineCompile(String[]), which this method simply wraps 
+     * error call commandLineCompile(String[]), which this method simply wraps
+     *
+     * @param args command line arguments
      */
     public static void main(String[] args) {
+        commandLineCompileWithErrorHandling(args, true);
+    }
 
+    /**
+     * Primary entry point for compiling from the command line
+     * (using the groovyc script).
+     * <p>
+     * If calling inside a process and you don't want the JVM to exit on an
+     * error call commandLineCompile(String[]), which this method simply wraps
+     *
+     * @param args               command line arguments
+     * @param lookupUnnamedFiles do a lookup for .groovy files not part of
+     *                           the given list of files to compile
+     */
+    public static void commandLineCompileWithErrorHandling(String[] args, boolean lookupUnnamedFiles) {
         try {
-            commandLineCompile(args);
-        } catch( Throwable e ) {
-            new ErrorReporter( e, displayStackTraceOnError).write( System.err );
+            commandLineCompile(args, lookupUnnamedFiles);
+        } catch (Throwable e) {
+            new ErrorReporter(e, displayStackTraceOnError).write(System.err);
             System.exit(1);
         }
     }
 
     public static void doCompilation(CompilerConfiguration configuration, CompilationUnit unit, String[] filenames) throws Exception {
+        doCompilation(configuration, unit, filenames, true);
+    }
+
+    public static void doCompilation(CompilerConfiguration configuration, CompilationUnit unit, String[] filenames, boolean lookupUnnamedFiles) throws Exception {
         File tmpDir = null;
         // if there are any joint compilation options set stubDir if not set
         try {
             if ((configuration.getJointCompilationOptions() != null)
                 && !configuration.getJointCompilationOptions().containsKey("stubDir"))
             {
-                tmpDir = createTempDir();
+                tmpDir = DefaultGroovyStaticMethods.createTempDir(null, "groovy-generated-", "-java-source");
                 configuration.getJointCompilationOptions().put("stubDir", tmpDir);
             }
             FileSystemCompiler compiler = new FileSystemCompiler(configuration, unit);
+            if (lookupUnnamedFiles) {
+                for (String filename : filenames) {
+                    File file = new File(filename);
+                    if (file.isFile()) {
+                        URL url = file.getAbsoluteFile().getParentFile().toURI().toURL();
+                        compiler.unit.getClassLoader().addURL(url);
+                    }
+                }
+            } else {
+                compiler.unit.getClassLoader().setResourceLoader(new GroovyResourceLoader() {
+                    public URL loadGroovySource(String filename) throws MalformedURLException {
+                        return null;
+                    }
+                });
+            }
             compiler.compile(filenames);
         } finally {
             try {
@@ -193,15 +240,25 @@ public class FileSystemCompiler {
         boolean errors = false;
         for (String filename : filenames) {
             if (filename.startsWith("@")) {
+                String fn = filename.substring(1);
+                BufferedReader br = null;
                 try {
-                    BufferedReader br = new BufferedReader(new FileReader(filename.substring(1)));
-                    String file;
-                    while ((file = br.readLine()) != null) {
+                    br = new BufferedReader(new FileReader(fn));
+                    for (String file; (file = br.readLine()) != null; ) {
                         fileList.add(file);
                     }
                 } catch (IOException ioe) {
-                    System.err.println("error: file not readable: " + filename.substring(1));
+                    System.err.println("error: file not readable: " + fn);
                     errors = true;
+                } finally {
+                    if (null != br) {
+                        try {
+                            br.close();
+                        } catch (IOException e) {
+                            System.err.println("error: failed to close buffered reader: " + fn);
+                            errors = true;
+                        }
+                    }
                 }
             } else {
                 fileList.add(filename);
@@ -214,7 +271,7 @@ public class FileSystemCompiler {
         }
     }
 
-    public static CompilerConfiguration generateCompilerConfigurationFromOptions(CommandLine cli) {
+    public static CompilerConfiguration generateCompilerConfigurationFromOptions(CommandLine cli) throws IOException {
         //
         // Setup the configuration data
 
@@ -232,6 +289,10 @@ public class FileSystemCompiler {
             configuration.setSourceEncoding(cli.getOptionValue("encoding"));
         }
 
+        if (cli.hasOption("basescript")) {
+            configuration.setScriptBaseClass(cli.getOptionValue("basescript"));
+        }
+
         // joint compilation parameters
         if (cli.hasOption('j')) {
             Map<String, Object> compilerOptions = new HashMap<String, Object>();
@@ -244,6 +305,27 @@ public class FileSystemCompiler {
 
             configuration.setJointCompilationOptions(compilerOptions);
         }
+
+        if (cli.hasOption("indy")) {
+            configuration.getOptimizationOptions().put("int", false);
+            configuration.getOptimizationOptions().put("indy", true);
+        }
+
+        if (cli.hasOption("configscript")) {
+            String path = cli.getOptionValue("configscript");
+            File groovyConfigurator = new File(path);
+            Binding binding = new Binding();
+            binding.setVariable("configuration", configuration);
+
+            CompilerConfiguration configuratorConfig = new CompilerConfiguration();
+            ImportCustomizer customizer = new ImportCustomizer();
+            customizer.addStaticStars("org.codehaus.groovy.control.customizers.builder.CompilerCustomizationBuilder");
+            configuratorConfig.addCompilationCustomizers(customizer);
+
+            GroovyShell shell = new GroovyShell(binding, configuratorConfig);
+            shell.evaluate(groovyConfigurator);
+        }
+        
         return configuration;
     }
 
@@ -265,6 +347,7 @@ public class FileSystemCompiler {
         options.addOption(OptionBuilder.withLongOpt("version").withDescription("Print the version").create('v'));
         options.addOption(OptionBuilder.withLongOpt("exception").withDescription("Print stack trace on error").create('e'));
         options.addOption(OptionBuilder.withLongOpt("jointCompilation").withDescription("Attach javac compiler to compile .java files").create('j'));
+        options.addOption(OptionBuilder.withLongOpt("basescript").hasArg().withArgName("class").withDescription("Base class name for scripts (must derive from Script)").create('b'));
 
         options.addOption(
                 OptionBuilder.withArgName("property=value")
@@ -277,43 +360,21 @@ public class FileSystemCompiler {
                         .hasArg()
                         .withDescription("passed to javac for joint compilation")
                         .create("F"));
+
+        options.addOption(OptionBuilder.withLongOpt("indy").withDescription("enables compilation using invokedynamic").create());
+        options.addOption(OptionBuilder.withLongOpt("configscript").hasArg().withDescription("A script for tweaking the configuration options").create());
         return options;
     }
 
+    /**
+     * Creates a temporary directory in the default temporary directory (as specified by the system
+     * property <i>java.io.tmpdir</i>.
+     *
+     * @deprecated Use {@link DefaultGroovyStaticMethods#createTempDir(java.io.File, String, String)} instead.
+     */
+    @Deprecated
     public static File createTempDir() throws IOException {
-        final int MAXTRIES = 3;
-        int accessDeniedCounter = 0;
-        File tempFile=null;
-        for (int i=0; i<MAXTRIES; i++) {
-            try {
-                tempFile = File.createTempFile("groovy-generated-", "-java-source");
-                tempFile.delete();
-                tempFile.mkdirs();
-                break;
-            } catch (IOException ioe) {
-                if (ioe.getMessage().startsWith("Access is denied")) {
-                    accessDeniedCounter++;
-                    try { Thread.sleep(100); } catch (InterruptedException e) {}
-                }
-                if (i==MAXTRIES-1) {
-                    if (accessDeniedCounter==MAXTRIES) {
-                        String msg = 
-                            "Access is denied.\nWe tried " +
-                            + accessDeniedCounter+
-                            " times to create a temporary directory"+
-                            " and failed each time. If you are on Windows"+
-                            " you are possibly victim to"+
-                            " http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6325169. "+
-                            " this is no bug in Groovy.";
-                        throw new IOException(msg);
-                    } else {
-                        throw ioe;
-                    }
-                }
-                continue;
-            }
-        }
-        return tempFile;
+        return DefaultGroovyStaticMethods.createTempDir(null);
     }
 
     public static void deleteRecursive(File file) {
