@@ -1,44 +1,63 @@
 /*
- * Copyright 2003-2011 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.codehaus.groovy.tools.javac;
 
-import org.codehaus.groovy.ast.*;
+import org.codehaus.groovy.ast.AnnotatedNode;
+import org.codehaus.groovy.ast.AnnotationNode;
+import org.codehaus.groovy.ast.ClassHelper;
+import org.codehaus.groovy.ast.ClassNode;
+import org.codehaus.groovy.ast.ConstructorNode;
+import org.codehaus.groovy.ast.FieldNode;
+import org.codehaus.groovy.ast.GenericsType;
+import org.codehaus.groovy.ast.ImportNode;
+import org.codehaus.groovy.ast.InnerClassNode;
+import org.codehaus.groovy.ast.MethodNode;
+import org.codehaus.groovy.ast.ModuleNode;
+import org.codehaus.groovy.ast.Parameter;
+import org.codehaus.groovy.ast.PropertyNode;
 import org.codehaus.groovy.ast.expr.ArgumentListExpression;
 import org.codehaus.groovy.ast.expr.ClassExpression;
+import org.codehaus.groovy.ast.expr.ClosureExpression;
 import org.codehaus.groovy.ast.expr.ConstantExpression;
 import org.codehaus.groovy.ast.expr.ConstructorCallExpression;
 import org.codehaus.groovy.ast.expr.Expression;
 import org.codehaus.groovy.ast.expr.ListExpression;
-import org.codehaus.groovy.ast.expr.VariableExpression;
 import org.codehaus.groovy.ast.expr.PropertyExpression;
-import org.codehaus.groovy.ast.expr.ClosureExpression;
+import org.codehaus.groovy.ast.expr.VariableExpression;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.ExpressionStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
 import org.codehaus.groovy.classgen.Verifier;
 import org.codehaus.groovy.control.ResolveVisitor;
+import org.codehaus.groovy.runtime.InvokerHelper;
 import org.codehaus.groovy.tools.Utilities;
+import org.codehaus.groovy.transform.trait.Traits;
 import org.objectweb.asm.Opcodes;
 
+import java.io.BufferedOutputStream;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
-import java.io.IOException;
+import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.io.StringWriter;
+import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
@@ -48,6 +67,7 @@ import java.util.Map;
 
 public class JavaStubGenerator {
     private boolean java5 = false;
+	private String encoding;
     private boolean requireSuperResolved = false;
     private File outputPath;
     private List<String> toCompile = new ArrayList<String>();
@@ -56,24 +76,26 @@ public class JavaStubGenerator {
     private ArrayList<ConstructorNode> constructors = new ArrayList<ConstructorNode>();
     private ModuleNode currentModule;
 
-    public JavaStubGenerator(final File outputPath, final boolean requireSuperResolved, final boolean java5) {
+    public JavaStubGenerator(final File outputPath, final boolean requireSuperResolved, final boolean java5, String encoding) {
         this.outputPath = outputPath;
         this.requireSuperResolved = requireSuperResolved;
         this.java5 = java5;
+		this.encoding = encoding;
         outputPath.mkdirs();
     }
 
     public JavaStubGenerator(final File outputPath) {
-        this(outputPath, false, false);
+        this(outputPath, false, false, Charset.defaultCharset().name());
     }
 
-    private void mkdirs(File parent, String relativeFile) {
+    private static void mkdirs(File parent, String relativeFile) {
         int index = relativeFile.lastIndexOf('/');
         if (index == -1) return;
         File dir = new File(parent, relativeFile.substring(0, index));
         dir.mkdirs();
     }
 
+    private static final int DEFAULT_BUFFER_SIZE = 32 * 1024; // 32K
     public void generateClass(ClassNode classNode) throws FileNotFoundException {
         // Only attempt to render our self if our super-class is resolved, else wait for it
         if (requireSuperResolved && !classNode.getSuperClass().isResolved()) {
@@ -92,8 +114,17 @@ public class JavaStubGenerator {
         toCompile.add(fileName);
 
         File file = new File(outputPath, fileName + ".java");
-        FileOutputStream fos = new FileOutputStream(file);
-        PrintWriter out = new PrintWriter(fos);
+        Charset charset = Charset.forName(encoding);
+        PrintWriter out =
+                new PrintWriter(
+                        new OutputStreamWriter(
+                                new BufferedOutputStream(
+                                        new FileOutputStream(file),
+                                        DEFAULT_BUFFER_SIZE
+                                ),
+                                charset
+                        )
+                );
 
         try {
             String packageName = classNode.getPackageName();
@@ -110,11 +141,6 @@ public class JavaStubGenerator {
             } catch (Exception e) {
                 // ignore
             }
-            try {
-                fos.close();
-            } catch (IOException e) {
-                // ignore
-            }
         }
     }
 
@@ -125,6 +151,21 @@ public class JavaStubGenerator {
         }
         try {
             Verifier verifier = new Verifier() {
+                @Override
+                public void visitClass(final ClassNode node) {
+                    List<Statement> savedStatements = new ArrayList<Statement>(node.getObjectInitializerStatements());
+                    super.visitClass(node);
+                    node.getObjectInitializerStatements().addAll(savedStatements);
+                    for (ClassNode inode : node.getAllInterfaces()) {
+                        if (Traits.isTrait(inode)) {
+                            List<PropertyNode> traitProps = inode.getProperties();
+                            for (PropertyNode pn : traitProps) {
+                                visitProperty(pn);
+                            }
+                        }
+                    }
+                }
+
                 public void addCovariantMethods(ClassNode cn) {}
                 protected void addTimeStamp(ClassNode node) {}
                 protected void addInitialization(ClassNode node) {}
@@ -178,12 +219,13 @@ public class JavaStubGenerator {
             verifier.visitClass(classNode);
             currentModule = classNode.getModule();
 
-            boolean isInterface = classNode.isInterface();
-            boolean isEnum = (classNode.getModifiers() & Opcodes.ACC_ENUM) != 0;
+            boolean isInterface = isInterfaceOrTrait(classNode);
+            boolean isEnum = classNode.isEnum();
             boolean isAnnotationDefinition = classNode.isAnnotationDefinition();
             printAnnotations(out, classNode);
             printModifiers(out, classNode.getModifiers()
-                    & ~(isInterface ? Opcodes.ACC_ABSTRACT : 0));
+                    & ~(isInterface ? Opcodes.ACC_ABSTRACT : 0)
+                    & ~(isEnum ? Opcodes.ACC_FINAL | Opcodes.ACC_ABSTRACT : 0));
 
             if (isInterface) {
                 if (isAnnotationDefinition) {
@@ -266,6 +308,45 @@ public class JavaStubGenerator {
             }
             printMethod(out, classNode, method);
         }
+
+        for (ClassNode node : classNode.getAllInterfaces()) {
+            if (Traits.isTrait(node)) {
+                List<MethodNode> traitMethods = node.getMethods();
+                for (MethodNode traitMethod : traitMethods) {
+                    MethodNode method = classNode.getMethod(traitMethod.getName(), traitMethod.getParameters());
+                    if (method == null) {
+                        for (MethodNode methodNode : propertyMethods) {
+                            if (methodNode.getName().equals(traitMethod.getName())) {
+                                boolean sameParams = sameParameterTypes(methodNode);
+                                if (sameParams) {
+                                    method = methodNode;
+                                    break;
+                                }
+                            }
+                        }
+                        if (method==null) {
+                            printMethod(out, classNode, traitMethod);
+                        }
+                    }
+                }
+            }
+        }
+
+    }
+
+    private static boolean sameParameterTypes(final MethodNode methodNode) {
+        Parameter[] a = methodNode.getParameters();
+        Parameter[] b = methodNode.getParameters();
+        boolean sameParams = a.length == b.length;
+        if (sameParams) {
+            for (int i = 0; i < a.length; i++) {
+                if (!a[i].getType().equals(b[i].getType())) {
+                    sameParams = false;
+                    break;
+                }
+            }
+        }
+        return sameParams;
     }
 
     private void printConstructors(PrintWriter out, ClassNode classNode) {
@@ -280,7 +361,7 @@ public class JavaStubGenerator {
     }
 
     private void printFields(PrintWriter out, ClassNode classNode) {
-        boolean isInterface = classNode.isInterface();
+        boolean isInterface = isInterfaceOrTrait(classNode);
         List<FieldNode> fields = classNode.getFields();
         if (fields == null) return;
         List<FieldNode> enumFields = new ArrayList<FieldNode>(fields.size());
@@ -299,17 +380,17 @@ public class JavaStubGenerator {
         }
     }
 
-
-    private void printEnumFields(PrintWriter out, List<FieldNode> fields) {
-        if (fields.size() == 0) return;
-        boolean first = true;
-        for (FieldNode field : fields) {
-            if (!first) {
-                out.print(", ");
-            } else {
-                first = false;
+    private static void printEnumFields(PrintWriter out, List<FieldNode> fields) {
+        if (!fields.isEmpty()) {
+            boolean first = true;
+            for (FieldNode field : fields) {
+                if (!first) {
+                    out.print(", ");
+                } else {
+                    first = false;
+                }
+                out.print(field.getName());
             }
-            out.print(field.getName());
         }
         out.println(";");
     }
@@ -326,9 +407,30 @@ public class JavaStubGenerator {
 
         out.print(" ");
         out.print(fieldNode.getName());
-        if (isInterface) {
+        if (isInterface || (fieldNode.getModifiers() & Opcodes.ACC_FINAL) != 0) {
             out.print(" = ");
-            if (ClassHelper.isPrimitiveType(type)) {
+            Expression valueExpr = fieldNode.getInitialValueExpression();
+            if (valueExpr instanceof ConstantExpression) {
+                valueExpr = Verifier.transformToPrimitiveConstantIfPossible((ConstantExpression) valueExpr);
+            }
+            if (valueExpr instanceof ConstantExpression
+                    && fieldNode.isStatic() && fieldNode.isFinal()
+                    && ClassHelper.isStaticConstantInitializerType(valueExpr.getType())
+                    && valueExpr.getType().equals(fieldNode.getType())) {
+                // GROOVY-5150 : Initialize value with a dummy constant so that Java cross compiles correctly
+                if (ClassHelper.STRING_TYPE.equals(valueExpr.getType())) {
+                    out.print(formatString(valueExpr.getText()));
+                } else if (ClassHelper.char_TYPE.equals(valueExpr.getType())) {
+                    out.print("'"+valueExpr.getText()+"'");
+                } else {
+                    ClassNode constantType = valueExpr.getType();
+                    out.print('(');
+                    printType(out, type);
+                    out.print(") ");
+                    out.print(valueExpr.getText());
+                    if (ClassHelper.Long_TYPE.equals(ClassHelper.getWrapper(constantType))) out.print('L');
+                }
+            } else if (ClassHelper.isPrimitiveType(type)) {
                 String val = type == ClassHelper.boolean_TYPE ? "false" : "0";
                 out.print("new " + ClassHelper.getWrapper(type) + "((" + type + ")" + val + ")");
             } else {
@@ -338,14 +440,22 @@ public class JavaStubGenerator {
         out.println(";");
     }
 
-    private ConstructorCallExpression getConstructorCallExpression(ConstructorNode constructorNode) {
+    private static String formatChar(String ch) {
+        return "'" + escapeSpecialChars("" + ch.charAt(0)) + "'";
+    }
+
+    private static String formatString(String s) {
+        return "\"" + escapeSpecialChars(s) + "\"";
+    }
+
+    private static ConstructorCallExpression getConstructorCallExpression(ConstructorNode constructorNode) {
         Statement code = constructorNode.getCode();
         if (!(code instanceof BlockStatement))
             return null;
 
         BlockStatement block = (BlockStatement) code;
         List stats = block.getStatements();
-        if (stats == null || stats.size() == 0)
+        if (stats == null || stats.isEmpty())
             return null;
 
         Statement stat = (Statement) stats.get(0);
@@ -381,16 +491,25 @@ public class JavaStubGenerator {
         }
     }
 
-    private Parameter[] selectAccessibleConstructorFromSuper(ConstructorNode node) {
+    private static Parameter[] selectAccessibleConstructorFromSuper(ConstructorNode node) {
         ClassNode type = node.getDeclaringClass();
-        ClassNode superType = type.getSuperClass();
+        ClassNode superType = type.getUnresolvedSuperClass();
 
+        Parameter[] bestMatch = null;
         for (ConstructorNode c : superType.getDeclaredConstructors()) {
             // Only look at things we can actually call
-            if (c.isPublic() || c.isProtected()) {
-                return c.getParameters();
+            if (!c.isPublic() && !c.isProtected()) continue;
+            Parameter[] parameters = c.getParameters();
+            // workaround for GROOVY-5859: remove generic type info
+            Parameter[] copy = new Parameter[parameters.length];
+            for (int i = 0; i < copy.length; i++) {
+                Parameter orig = parameters[i];
+                copy[i] = new Parameter(orig.getOriginType().getPlainNodeReference(), orig.getName());
             }
+            if (noExceptionToAvoid(node,c)) return copy;
+            if (bestMatch==null) bestMatch = copy;
         }
+        if (bestMatch!=null) return bestMatch;
 
         // fall back for parameterless constructor
         if (superType.isPrimaryClassNode()) {
@@ -400,6 +519,27 @@ public class JavaStubGenerator {
         return null;
     }
 
+    private static boolean noExceptionToAvoid(ConstructorNode fromStub, ConstructorNode fromSuper) {
+        ClassNode[] superExceptions = fromSuper.getExceptions();
+        if (superExceptions==null || superExceptions.length==0) return true;
+
+        ClassNode[] stubExceptions = fromStub.getExceptions();
+        if (stubExceptions==null || stubExceptions.length==0) return false;
+
+
+        // if all remaining exceptions are used in the stub we are good
+        outer: for (int i=0; i<superExceptions.length; i++) {
+            ClassNode superExc = superExceptions[i];
+            for (ClassNode stub:stubExceptions) {
+                if (stub.isDerivedFrom(superExc)) continue outer;
+            }
+            // not found 
+            return false;
+        }
+
+        return true;
+    }
+
     private void printSpecialConstructorArgs(PrintWriter out, ConstructorNode node, ConstructorCallExpression constrCall) {
         // Select a constructor from our class, or super-class which is legal to call,
         // then write out an invoke w/nulls using casts to avoid ambiguous crapo
@@ -457,7 +597,7 @@ public class JavaStubGenerator {
         out.println(");");
     }
 
-    private ClassNode getConstructorArgumentType(Expression arg, ConstructorNode node) {
+    private static ClassNode getConstructorArgumentType(Expression arg, ConstructorNode node) {
         if (!(arg instanceof VariableExpression)) return arg.getType();
         VariableExpression vexp = (VariableExpression) arg;
         String name = vexp.getName();
@@ -475,7 +615,13 @@ public class JavaStubGenerator {
         if (methodNode.isSynthetic() && methodNode.getName().equals("$getStaticMetaClass")) return;
 
         printAnnotations(out, methodNode);
-        if (!clazz.isInterface()) printModifiers(out, methodNode.getModifiers());
+        if (!isInterfaceOrTrait(clazz)) {
+            int modifiers = methodNode.getModifiers();
+            if (isDefaultTraitImpl(methodNode)) {
+                modifiers ^= Opcodes.ACC_ABSTRACT;
+            }
+            printModifiers(out, modifiers & ~(clazz.isEnum() ? Opcodes.ACC_ABSTRACT : 0));
+        }
 
         printGenericsBounds(out, methodNode.getGenericsTypes());
         out.print(" ");
@@ -496,7 +642,32 @@ public class JavaStubGenerator {
             printType(out, exception);
         }
 
-        if ((methodNode.getModifiers() & Opcodes.ACC_ABSTRACT) != 0) {
+        if (Traits.isTrait(clazz)) {
+            out.println(";");
+        } else if (isAbstract(methodNode) && !clazz.isEnum()) {
+            if (clazz.isAnnotationDefinition() && methodNode.hasAnnotationDefault()) {
+                Statement fs = methodNode.getFirstStatement();
+                if (fs instanceof ExpressionStatement) {
+                    ExpressionStatement es = (ExpressionStatement) fs;
+                    Expression re = es.getExpression();
+                    out.print(" default ");
+                    ClassNode rt = methodNode.getReturnType();
+                    boolean classReturn = ClassHelper.CLASS_Type.equals(rt) || (rt.isArray() && ClassHelper.CLASS_Type.equals(rt.getComponentType()));
+                    if (re instanceof ListExpression) {
+                        out.print("{ ");
+                        ListExpression le = (ListExpression) re;
+                        boolean first = true;
+                        for (Expression expression : le.getExpressions()) {
+                            if (first) first = false;
+                            else out.print(", ");
+                            printValue(out, expression, classReturn);
+                        }
+                        out.print(" }");
+                    } else {
+                        printValue(out, re, classReturn);
+                    }
+                }
+            }
             out.println(";");
         } else {
             out.print(" { ");
@@ -506,6 +677,54 @@ public class JavaStubGenerator {
         }
     }
 
+    private static boolean isAbstract(final MethodNode methodNode) {
+        if (isDefaultTraitImpl(methodNode)) {
+            return false;
+        }
+        if ((methodNode.getModifiers() & Opcodes.ACC_ABSTRACT) != 0) {
+            return true;
+        }
+        return false;
+    }
+
+    private static boolean isDefaultTraitImpl(final MethodNode methodNode) {
+        return Traits.isTrait(methodNode.getDeclaringClass()) && Traits.hasDefaultImplementation(methodNode);
+    }
+
+    private static void printValue(PrintWriter out, Expression re, boolean assumeClass) {
+        if (assumeClass) {
+            if (re.getType().getName().equals("groovy.lang.Closure")) {
+                out.print("groovy.lang.Closure.class");
+                return;
+            }
+            String className = re.getText();
+            out.print(className);
+            if (!className.endsWith(".class")) {
+                out.print(".class");
+            }
+        } else {
+            if (re instanceof ConstantExpression) {
+                ConstantExpression ce = (ConstantExpression) re;
+                Object value = ce.getValue();
+                if (ClassHelper.STRING_TYPE.equals(ce.getType())) {
+                    out.print(formatString((String)value));
+                } else if (ClassHelper.char_TYPE.equals(ce.getType()) || ClassHelper.Character_TYPE.equals(ce.getType())) {
+                    out.print(formatChar(value.toString()));
+                } else if (ClassHelper.long_TYPE.equals(ce.getType())) {
+                    out.print("" + value + "L");
+                } else if (ClassHelper.float_TYPE.equals(ce.getType())) {
+                    out.print("" + value + "f");
+                } else if (ClassHelper.double_TYPE.equals(ce.getType())) {
+                    out.print("" + value + "d");
+                } else {
+                    out.print(re.getText());
+                }
+            } else {
+                out.print(re.getText());
+            }
+        }
+    }
+
     private void printReturn(PrintWriter out, ClassNode retType) {
         String retName = retType.getName();
         if (!retName.equals("void")) {
@@ -584,7 +803,7 @@ public class JavaStubGenerator {
         }
     }
 
-    private void printGenericsBounds(PrintWriter out, GenericsType[] genericsTypes) {
+    private static void printGenericsBounds(PrintWriter out, GenericsType[] genericsTypes) {
         if (genericsTypes == null || genericsTypes.length == 0) return;
         out.print('<');
         for (int i = 0; i < genericsTypes.length; i++) {
@@ -601,6 +820,7 @@ public class JavaStubGenerator {
             int lastIndex = parameters.length - 1;
             boolean vararg = parameters[lastIndex].getType().isArray();
             for (int i = 0; i != parameters.length; ++i) {
+                printAnnotations(out, parameters[i]);
                 if (i == lastIndex && vararg) {
                     printType(out, parameters[i].getType().getComponentType());
                     out.print("...");
@@ -625,13 +845,14 @@ public class JavaStubGenerator {
     }
 
     private void printAnnotation(PrintWriter out, AnnotationNode annotation) {
-        out.print("@" + annotation.getClassNode().getName() + "(");
+        out.print("@" + annotation.getClassNode().getName().replace('$', '.') + "(");
         boolean first = true;
         Map<String, Expression> members = annotation.getMembers();
-        for (String key : members.keySet()) {
+        for (Map.Entry<String, Expression> entry : members.entrySet()) {
+            String key = entry.getKey();
             if (first) first = false;
             else out.print(", ");
-            out.print(key + "=" + getAnnotationValue(members.get(key)));
+            out.print(key + "=" + getAnnotationValue(entry.getValue()).replace('$', '.'));
         }
         out.print(") ");
     }
@@ -660,10 +881,16 @@ public class JavaStubGenerator {
             } else if (constValue instanceof Number || constValue instanceof Boolean)
                 val = constValue.toString();
             else
-                val = "\"" + escapeStringAnnotationValue(constValue.toString()) + "\"";
-        } else if (memberValue instanceof PropertyExpression || memberValue instanceof VariableExpression) {
+                val = "\"" + escapeSpecialChars(constValue.toString()) + "\"";
+        } else if (memberValue instanceof PropertyExpression) {
             // assume must be static class field or enum value or class that Java can resolve
             val = ((Expression) memberValue).getText();
+        } else if (memberValue instanceof VariableExpression) {
+            val = ((Expression) memberValue).getText();
+            //check for an alias
+            ImportNode alias = currentModule.getStaticImports().get(val);
+            if (alias != null)
+                val = alias.getClassName() + "." + alias.getFieldName();
         } else if (memberValue instanceof ClosureExpression) {
             // annotation closure; replaced with this specific class literal to cover the
             // case where annotation type uses Class<? extends Closure> for the closure's type
@@ -674,7 +901,7 @@ public class JavaStubGenerator {
         return val;
     }
 
-    private void printModifiers(PrintWriter out, int modifiers) {
+    private static void printModifiers(PrintWriter out, int modifiers) {
         if ((modifiers & Opcodes.ACC_PUBLIC) != 0)
             out.print("public ");
 
@@ -690,11 +917,14 @@ public class JavaStubGenerator {
         if ((modifiers & Opcodes.ACC_SYNCHRONIZED) != 0)
             out.print("synchronized ");
 
+        if ((modifiers & Opcodes.ACC_FINAL) != 0)
+            out.print("final ");
+
         if ((modifiers & Opcodes.ACC_ABSTRACT) != 0)
             out.print("abstract ");
     }
 
-    private void printImports(PrintWriter out, ClassNode classNode) {
+    private static void printImports(PrintWriter out, ClassNode classNode) {
         List<String> imports = new ArrayList<String>();
 
         ModuleNode moduleNode = classNode.getModule();
@@ -709,6 +939,15 @@ public class JavaStubGenerator {
 
         imports.addAll(Arrays.asList(ResolveVisitor.DEFAULT_IMPORTS));
 
+        for (Map.Entry<String, ImportNode> entry : moduleNode.getStaticImports().entrySet()) {
+            if (entry.getKey().equals(entry.getValue().getFieldName()))
+                imports.add("static "+entry.getValue().getType().getName()+"."+entry.getKey());
+        }
+
+        for (Map.Entry<String, ImportNode> entry : moduleNode.getStaticStarImports().entrySet()) {
+            imports.add("static "+entry.getValue().getType().getName()+".");
+        }
+
         for (String imp : imports) {
             String s = new StringBuilder()
                     .append("import ")
@@ -727,7 +966,12 @@ public class JavaStubGenerator {
         }
     }
 
-    private static String escapeStringAnnotationValue(String value) {
-        return value.replace("\n", "\\n").replace("\r", "\\r").replace("\"", "\\\"");
+    private static String escapeSpecialChars(String value) {
+        return InvokerHelper.escapeBackslashes(value).replace("\"", "\\\"");
+
+    }
+
+    private static boolean isInterfaceOrTrait(ClassNode cn) {
+        return cn.isInterface() || Traits.isTrait(cn);
     }
 }
