@@ -1,17 +1,20 @@
 /*
- * Copyright 2003-2013 the original author or authors.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.codehaus.groovy.transform.trait;
 
@@ -55,7 +58,6 @@ import org.codehaus.groovy.syntax.Types;
 import org.codehaus.groovy.transform.ASTTransformationCollectorCodeVisitor;
 import org.codehaus.groovy.transform.AbstractASTTransformation;
 import org.codehaus.groovy.transform.GroovyASTTransformation;
-import org.codehaus.groovy.transform.sc.StaticCompileTransformation;
 
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
@@ -66,10 +68,17 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Set;
 
+import static org.codehaus.groovy.ast.tools.GeneralUtils.returnS;
+import static org.codehaus.groovy.transform.trait.SuperCallTraitTransformer.UNRESOLVED_HELPER_CLASS;
+
 /**
- * Handles generation of code for the @Trait annotation. A class annotated with @Trait will generate, instead: <ul>
- * <li>an <i>interface</i> with the same name</li> <li>an utility inner class that will be used by the compiler to
- * handle the trait</li> </ul>
+ * Handles generation of code for the traits (trait keyword is equivalent to using the @Trait annotation).
+ * A class annotated with @Trait will generate, instead:
+ * <ul>
+ * <li>an <i>interface</i> with the same name</li>
+ * <li>a utility inner class that will be used by the compiler to implement the trait</li>
+ * <li>potentially a utility inner class to assist with implementing trait fields</li>
+ * </ul>
  *
  * @author Cedric Champeau
  */
@@ -99,7 +108,21 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
             checkExtendsClause(cNode);
             generateMethodsWithDefaultArgs(cNode);
             replaceExtendsByImplements(cNode);
-            createHelperClass(cNode);
+            ClassNode helperClassNode = createHelperClass(cNode);
+            resolveHelperClassIfNecessary(helperClassNode);
+        }
+    }
+
+    private void resolveHelperClassIfNecessary(final ClassNode helperClassNode) {
+        if (helperClassNode == null) {
+            return;
+        }
+        for (ClassNode cNode : unit.getAST().getClasses()) {
+            ClassNode unresolvedHelperNode = cNode.getNodeMetaData(UNRESOLVED_HELPER_CLASS);
+            if (unresolvedHelperNode != null
+                    && unresolvedHelperNode.getName().equals(helperClassNode.getName())) {
+                unresolvedHelperNode.setRedirect(helperClassNode);
+            }
         }
     }
 
@@ -139,7 +162,7 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
         }
     }
 
-    private void createHelperClass(final ClassNode cNode) {
+    private ClassNode createHelperClass(final ClassNode cNode) {
         ClassNode helper = new InnerClassNode(
                 cNode,
                 Traits.helperClassName(cNode),
@@ -186,9 +209,9 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
                 if (!methodNode.isSynthetic() && (methodNode.isProtected() || methodNode.getModifiers()==0)) {
                     unit.addError(new SyntaxException("Cannot have protected/package private method in a trait (" + cNode.getName() + "#" + methodNode.getTypeDescriptor() + ")",
                             methodNode.getLineNumber(), methodNode.getColumnNumber()));
-                    return;
+                    return null;
                 }
-                helper.addMethod(processMethod(cNode, methodNode, fieldHelper, fieldNames));
+                helper.addMethod(processMethod(cNode, helper, methodNode, fieldHelper, fieldNames));
                 if (methodNode.isPrivate() || methodNode.isStatic()) {
                     nonPublicAPIMethods.add(methodNode);
                 }
@@ -202,7 +225,7 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
 
         // add fields
         for (FieldNode field : fields) {
-            processField(field, initializer, staticInitializer, fieldHelper, cNode, fieldNames);
+            processField(field, initializer, staticInitializer, fieldHelper, helper, cNode, fieldNames);
         }
 
         // clear properties to avoid generation of methods
@@ -217,7 +240,7 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
         }
 
         // visit AST xforms
-        registerASTTranformations(helper);
+        registerASTTransformations(helper);
 
         unit.getAST().addClass(helper);
         if (fieldHelper != null) {
@@ -229,6 +252,7 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
         if (fieldHelper!=null) {
             resolveScope(fieldHelper);
         }
+        return helper;
     }
 
     private MethodNode createInitMethod(final boolean isStatic, final ClassNode cNode, final ClassNode helper) {
@@ -250,7 +274,7 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
         return initializer;
     }
 
-    private void registerASTTranformations(final ClassNode helper) {
+    private void registerASTTransformations(final ClassNode helper) {
         ASTTransformationCollectorCodeVisitor collector = new ASTTransformationCollectorCodeVisitor(
                 unit, compilationUnit.getTransformLoader()
         );
@@ -382,14 +406,33 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
         return true;
     }
 
+    private void processField(final FieldNode field, final MethodNode initializer, final MethodNode staticInitializer,
+                              final ClassNode fieldHelper, final ClassNode helper, final ClassNode trait,
+                              final Set<String> knownFields) {
+        if (field.isProtected()) {
+            unit.addError(new SyntaxException("Cannot have protected field in a trait (" + trait.getName() + "#" + field.getName() + ")",
+                    field.getLineNumber(), field.getColumnNumber()));
+            return;
+        }
 
-    private void processField(final FieldNode field, final MethodNode initializer, final MethodNode staticInitializer, final ClassNode fieldHelper, final ClassNode trait, final Set<String> knownFields) {
         Expression initialExpression = field.getInitialExpression();
         MethodNode selectedMethod = field.isStatic()?staticInitializer:initializer;
         if (initialExpression != null) {
             VariableExpression thisObject = new VariableExpression(selectedMethod.getParameters()[0]);
             ExpressionStatement initCode = new ExpressionStatement(initialExpression);
-            processBody(thisObject, selectedMethod, initCode, trait, fieldHelper, knownFields);
+            processBody(thisObject, initCode, trait, helper, fieldHelper, knownFields);
+            if (field.isFinal()) {
+                String baseName = field.isStatic() ? Traits.STATIC_INIT_METHOD : Traits.INIT_METHOD;
+                MethodNode fieldInitializer = new MethodNode(
+                        baseName + Traits.remappedFieldName(trait, field.getName()),
+                        ACC_STATIC | ACC_PUBLIC | ACC_SYNTHETIC,
+                        field.getOriginType(),
+                        new Parameter[]{createSelfParameter(trait, field.isStatic())},
+                        ClassNode.EMPTY_ARRAY,
+                        returnS(initCode.getExpression())
+                );
+                helper.addMethod(fieldInitializer);
+            }
             BlockStatement code = (BlockStatement) selectedMethod.getCode();
             MethodCallExpression mce;
             if (field.isStatic()) {
@@ -413,7 +456,7 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
             mce.setSourcePosition(initialExpression);
             code.addStatement(new ExpressionStatement(mce));
         }
-        // define setter/getter helper methods
+        // define setter/getter helper methods (setter added even for final fields for legacy compatibility)
         fieldHelper.addMethod(
                 Traits.helperSetterName(field),
                 ACC_PUBLIC | ACC_ABSTRACT,
@@ -433,9 +476,8 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
 
         // dummy fields are only used to carry annotations if instance field
         // and to differentiate from static fields otherwise
-        String dummyFieldName = (field.isStatic() ? Traits.STATIC_FIELD_PREFIX : Traits.FIELD_PREFIX) +
-                (field.isPublic()? Traits.PUBLIC_FIELD_PREFIX : Traits.PRIVATE_FIELD_PREFIX)+
-                Traits.remappedFieldName(field.getOwner(), field.getName());
+        int mods = field.getModifiers() & Traits.FIELD_PREFIX_MASK;
+        String dummyFieldName = String.format("$0x%04x", mods) + Traits.remappedFieldName(field.getOwner(), field.getName());
         FieldNode dummyField = new FieldNode(
                 dummyFieldName,
                 ACC_STATIC | ACC_PUBLIC | ACC_FINAL | ACC_SYNTHETIC,
@@ -449,9 +491,27 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
         GeneralUtils.copyAnnotatedNodeAnnotations(field, copied, notCopied);
         dummyField.addAnnotations(copied);
         fieldHelper.addField(dummyField);
+
+        // retain legacy field (will be given lower precedence than above)
+        dummyFieldName = (field.isStatic() ? Traits.STATIC_FIELD_PREFIX : Traits.FIELD_PREFIX) +
+                (field.isPublic()? Traits.PUBLIC_FIELD_PREFIX : Traits.PRIVATE_FIELD_PREFIX)+
+                Traits.remappedFieldName(field.getOwner(), field.getName());
+        dummyField = new FieldNode(
+                dummyFieldName,
+                ACC_STATIC | ACC_PUBLIC | ACC_FINAL | ACC_SYNTHETIC,
+                field.getOriginType(),
+                fieldHelper,
+                null
+        );
+        // copy annotations from field to legacy dummy field
+        copied = new LinkedList<AnnotationNode>();
+        notCopied = new LinkedList<AnnotationNode>();
+        GeneralUtils.copyAnnotatedNodeAnnotations(field, copied, notCopied);
+        dummyField.addAnnotations(copied);
+        fieldHelper.addField(dummyField);
     }
 
-    private MethodNode processMethod(ClassNode traitClass, MethodNode methodNode, ClassNode fieldHelper, Collection<String> knownFields) {
+    private MethodNode processMethod(ClassNode traitClass, ClassNode traitHelperClass, MethodNode methodNode, ClassNode fieldHelper, Collection<String> knownFields) {
         Parameter[] initialParams = methodNode.getParameters();
         Parameter[] newParams = new Parameter[initialParams.length + 1];
         newParams[0] = createSelfParameter(traitClass, methodNode.isStatic());
@@ -463,7 +523,7 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
                 methodNode.getReturnType(),
                 newParams,
                 methodNode.getExceptions(),
-                processBody(new VariableExpression(newParams[0]), methodNode, methodNode.getCode(), traitClass, fieldHelper, knownFields)
+                processBody(new VariableExpression(newParams[0]), methodNode.getCode(), traitClass, traitHelperClass, fieldHelper, knownFields)
         );
         mNode.setSourcePosition(methodNode);
         mNode.addAnnotations(filterAnnotations(methodNode.getAnnotations()));
@@ -513,13 +573,13 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
         return type;
     }
 
-    private Statement processBody(VariableExpression thisObject, MethodNode methodNode, Statement code, ClassNode trait, ClassNode fieldHelper, Collection<String> knownFields) {
+    private Statement processBody(VariableExpression thisObject, Statement code, ClassNode trait, ClassNode traitHelper, ClassNode fieldHelper, Collection<String> knownFields) {
         if (code == null) return null;
         NAryOperationRewriter operationRewriter = new NAryOperationRewriter(unit, knownFields);
         code.visit(operationRewriter);
         SuperCallTraitTransformer superTrn = new SuperCallTraitTransformer(unit);
         code.visit(superTrn);
-        TraitReceiverTransformer trn = new TraitReceiverTransformer(thisObject, unit, trait, fieldHelper, knownFields);
+        TraitReceiverTransformer trn = new TraitReceiverTransformer(thisObject, unit, trait, traitHelper, fieldHelper, knownFields);
         code.visit(trn);
         return code;
     }
@@ -551,9 +611,11 @@ public class TraitASTTransformation extends AbstractASTTransformation implements
 
         @Override
         public Expression transform(final Expression exp) {
-            Expression replacement = exp.getNodeMetaData(TraitASTTransformation.POST_TYPECHECKING_REPLACEMENT);
-            if (replacement!=null) {
-                return replacement;
+            if (exp != null) {
+                Expression replacement = exp.getNodeMetaData(TraitASTTransformation.POST_TYPECHECKING_REPLACEMENT);
+                if (replacement!=null) {
+                    return replacement;
+                }
             }
             return super.transform(exp);
         }
